@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2005 MySQL AB
+ Copyright (C) 2002-2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -560,7 +560,14 @@ public class CallableStatement extends PreparedStatement implements
 		CallableStatementParam paramDescriptor = this.paramInfo
 				.getParameter(localParamIndex);
 
-		if (!paramDescriptor.isOut) {
+		// We don't have reliable metadata in this case, trust
+		// the caller
+		
+		if (this.connection.getNoAccessToProcedureBodies()) {
+			paramDescriptor.isOut = true;
+			paramDescriptor.isIn = true;
+			paramDescriptor.inOutModifier = DatabaseMetaData.procedureColumnInOut;
+		} else if (!paramDescriptor.isOut) {
 			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.9") + paramIndex //$NON-NLS-1$
 							+ Messages.getString("CallableStatement.10"), //$NON-NLS-1$
@@ -653,7 +660,7 @@ public class CallableStatement extends PreparedStatement implements
 			row[3] = StringUtils.s2b(String.valueOf(i), this.connection); // COLUMN_NAME
 
 			row[4] = StringUtils.s2b(String
-					.valueOf(DatabaseMetaData.procedureColumnInOut),
+					.valueOf(DatabaseMetaData.procedureColumnIn),
 					this.connection);
 
 			row[5] = StringUtils.s2b(String.valueOf(Types.VARCHAR),
@@ -701,7 +708,7 @@ public class CallableStatement extends PreparedStatement implements
 
 			paramTypesRs = dbmd.getProcedureColumns(this.connection
 					.versionMeetsMinimum(5, 0, 2)
-					& useCatalog ? this.currentCatalog : null, null, procName,
+					&& useCatalog ? this.currentCatalog : null, null, procName,
 					"%"); //$NON-NLS-1$
 
 			convertGetProcedureColumnsToInternalDescriptors(paramTypesRs);
@@ -823,13 +830,16 @@ public class CallableStatement extends PreparedStatement implements
 	}
 
 	private String extractProcedureName() throws SQLException {
+		String sanitizedSql = StringUtils.stripComments(this.originalSql, 
+				"`\"'", "`\"'", true, false, true, true);
+		
 		// TODO: Do this with less memory allocation
-		int endCallIndex = StringUtils.indexOfIgnoreCase(this.originalSql,
+		int endCallIndex = StringUtils.indexOfIgnoreCase(sanitizedSql,
 				"CALL "); //$NON-NLS-1$
 		int offset = 5;
 
 		if (endCallIndex == -1) {
-			endCallIndex = StringUtils.indexOfIgnoreCase(this.originalSql,
+			endCallIndex = StringUtils.indexOfIgnoreCase(sanitizedSql,
 					"SELECT ");
 			offset = 7;
 		}
@@ -837,7 +847,7 @@ public class CallableStatement extends PreparedStatement implements
 		if (endCallIndex != -1) {
 			StringBuffer nameBuf = new StringBuffer();
 
-			String trimmedStatement = this.originalSql.substring(
+			String trimmedStatement = sanitizedSql.substring(
 					endCallIndex + offset).trim();
 
 			int statementLength = trimmedStatement.length();
@@ -854,9 +864,9 @@ public class CallableStatement extends PreparedStatement implements
 
 			return nameBuf.toString();
 		}
+		
 		throw SQLError.createSQLException(Messages.getString("CallableStatement.1"), //$NON-NLS-1$
 				SQLError.SQL_STATE_GENERAL_ERROR);
-
 	}
 
 	/**
@@ -1955,8 +1965,8 @@ public class CallableStatement extends PreparedStatement implements
 						setPstmt = this.connection
 								.clientPrepareStatement(queryBuf.toString());
 
-						byte[] parameterAsBytes = this
-								.getBytesRepresentation(inParamInfo.index);
+						byte[] parameterAsBytes = getBytesRepresentation(
+								inParamInfo.index);
 
 						if (parameterAsBytes != null) {
 							if (parameterAsBytes.length > 8
@@ -1971,7 +1981,22 @@ public class CallableStatement extends PreparedStatement implements
 								setPstmt.setBytesNoEscapeNoQuotes(1,
 										parameterAsBytes);
 							} else {
-								setPstmt.setBytes(1, parameterAsBytes);
+								int sqlType = inParamInfo.desiredJdbcType;
+								
+								switch (sqlType) {
+								case Types.BIT:
+								case Types.BINARY: 
+								case Types.BLOB: 
+								case Types.JAVA_OBJECT:
+								case Types.LONGVARBINARY: 
+								case Types.VARBINARY:
+									setPstmt.setBytes(1, parameterAsBytes);
+									break;
+								default:
+									// the inherited PreparedStatement methods
+									// have already escaped and quoted these parameters
+									setPstmt.setBytesNoEscape(1, parameterAsBytes); 
+								}
 							}
 						} else {
 							setPstmt.setNull(1, Types.NULL);
