@@ -2,12 +2,12 @@
  Copyright (C) 2002-2004 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
- it under the terms of version 2 of the GNU General Public License as 
+ it under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.
 
- There are special exceptions to the terms and conditions of the GPL 
- as it is applied to this software. View the full text of the 
- exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
+ There are special exceptions to the terms and conditions of the GPL
+ as it is applied to this software. View the full text of the
+ exception in file EXCEPTIONS-CONNECTOR-J in the directory of this
  software distribution.
 
  This program is distributed in the hope that it will be useful,
@@ -30,21 +30,16 @@ import java.sql.Types;
 
 /**
  * Field is a class used to describe fields in a ResultSet
- * 
+ *
  * @author Mark Matthews
- * @version $Id: Field.java 5639 2006-08-16 14:18:23Z mmatthews $
+ * @version $Id: Field.java 6531 2007-08-30 15:21:06Z mmatthews $
  */
 public class Field {
-	// ~ Static fields/initializers
-	// ---------------------------------------------
 
 	private static final int AUTO_INCREMENT_FLAG = 512;
 
 	private static final int NO_CHARSET_INFO = -1;
 
-	// ~ Instance fields
-	// --------------------------------------------------------
-
 	private byte[] buffer;
 
 	private int charsetIndex = 0;
@@ -110,13 +105,12 @@ public class Field {
 	private int tableNameLength;
 
 	private int tableNameStart;
-	
+
 	private boolean useOldNameMetadata = false;
 
 	private boolean isSingleBit;
 
-	// ~ Constructors
-	// -----------------------------------------------------------
+	private int maxBytesPerChar;
 
 	/**
 	 * Constructor used when communicating with 4.1 and newer servers
@@ -156,16 +150,21 @@ public class Field {
 		// charset
 		this.charsetIndex = charsetIndex;
 
-		this.charsetName = this.connection
-				.getCharsetNameForIndex(this.charsetIndex);
-		
+
 		// Map MySqlTypes to java.sql Types
 		this.sqlType = MysqlDefs.mysqlToJavaType(this.mysqlType);
-		
+
+        checkForImplicitTemporaryTable();
 		// Re-map to 'real' blob type, if we're a BLOB
 
 		if (this.mysqlType == MysqlDefs.FIELD_TYPE_BLOB) {
-			if (this.charsetIndex == 63 || 
+		    boolean isFromFunction = this.originalTableNameLength == 0;
+
+		    if (this.connection != null && this.connection.getBlobsAreStrings() ||
+		            (this.connection.getFunctionsNeverReturnBlobs() && isFromFunction)) {
+		        this.sqlType = Types.VARCHAR;
+		        this.mysqlType = MysqlDefs.FIELD_TYPE_VARCHAR;
+		    } else if (this.charsetIndex == 63 ||
 					!this.connection.versionMeetsMinimum(4, 1, 0)) {
 				setBlobTypeBasedOnLength();
 				this.sqlType = MysqlDefs.mysqlToJavaType(this.mysqlType);
@@ -176,35 +175,6 @@ public class Field {
 			}
 		}
 
-		// Handle VARBINARY/BINARY (server doesn't have a different type
-		// for this
-
-		boolean isBinary = isBinary();
-
-		if (this.connection.versionMeetsMinimum(4, 1, 0) &&
-				this.mysqlType == MysqlDefs.FIELD_TYPE_VAR_STRING && 
-				isBinary &&
-				this.charsetIndex == 63) {
-			if (this.isOpaqueBinary()) {
-				this.sqlType = Types.VARBINARY;
-			}
-		} 
-		
-		if (this.connection.versionMeetsMinimum(4, 1, 0) &&
-				this.mysqlType == MysqlDefs.FIELD_TYPE_STRING && 
-				isBinary && this.charsetIndex == 63) {
-			//
-			// Okay, this is a hack, but there's currently no way
-			// to easily distinguish something like DATE_FORMAT( ..)
-			// from the "BINARY" column type, other than looking
-			// at the original column name.
-			//
-			
-			if (isOpaqueBinary()) {
-				this.sqlType = Types.BINARY;
-			}
-		}
-
 		if (this.sqlType == Types.TINYINT && this.length == 1
 				&& this.connection.getTinyInt1isBit()) {
 			// Adjust for pseudo-boolean
@@ -218,32 +188,70 @@ public class Field {
 
 		}
 
-		if (this.mysqlType == MysqlDefs.FIELD_TYPE_BIT) {
-			this.isSingleBit = (this.length == 0);
-			
-			if (this.connection != null && (this.connection.versionMeetsMinimum(5, 0, 21) ||
-					this.connection.versionMeetsMinimum(5, 1, 10)) && this.length == 1) {
-				this.isSingleBit = true;
-			}
-			
-			if (this.isSingleBit) {
-				this.sqlType = Types.BIT;
-			} else {
-				this.sqlType = Types.VARBINARY;
-				this.colFlag |= 128; // we need to pretend this is a full
-				this.colFlag |= 16; // binary blob
-			}
-		}
+		if (!isNativeNumericType() && !isNativeDateTimeType()) {
+			this.charsetName = this.connection
+				.getCharsetNameForIndex(this.charsetIndex);
 
-		
 
-		//
-		// Handle TEXT type (special case), Fix proposed by Peter McKeown
-		//
-		if ((this.sqlType == java.sql.Types.LONGVARBINARY) && !isBinary) {
-			this.sqlType = java.sql.Types.LONGVARCHAR;
-		} else if ((this.sqlType == java.sql.Types.VARBINARY) && !isBinary) {
-			this.sqlType = java.sql.Types.VARCHAR;
+			// Handle VARBINARY/BINARY (server doesn't have a different type
+			// for this
+
+			boolean isBinary = isBinary();
+
+			if (this.connection.versionMeetsMinimum(4, 1, 0) &&
+					this.mysqlType == MysqlDefs.FIELD_TYPE_VAR_STRING &&
+					isBinary &&
+					this.charsetIndex == 63) {
+				if (this.isOpaqueBinary()) {
+					this.sqlType = Types.VARBINARY;
+				}
+			}
+
+			if (this.connection.versionMeetsMinimum(4, 1, 0) &&
+					this.mysqlType == MysqlDefs.FIELD_TYPE_STRING &&
+					isBinary && this.charsetIndex == 63) {
+				//
+				// Okay, this is a hack, but there's currently no way
+				// to easily distinguish something like DATE_FORMAT( ..)
+				// from the "BINARY" column type, other than looking
+				// at the original column name.
+				//
+
+				if (isOpaqueBinary() && !this.connection.getBlobsAreStrings()) {
+					this.sqlType = Types.BINARY;
+				}
+			}
+
+
+
+			if (this.mysqlType == MysqlDefs.FIELD_TYPE_BIT) {
+				this.isSingleBit = (this.length == 0);
+
+				if (this.connection != null && (this.connection.versionMeetsMinimum(5, 0, 21) ||
+						this.connection.versionMeetsMinimum(5, 1, 10)) && this.length == 1) {
+					this.isSingleBit = true;
+				}
+
+				if (this.isSingleBit) {
+					this.sqlType = Types.BIT;
+				} else {
+					this.sqlType = Types.VARBINARY;
+					this.colFlag |= 128; // we need to pretend this is a full
+					this.colFlag |= 16; // binary blob
+					isBinary = true;
+				}
+			}
+
+			//
+			// Handle TEXT type (special case), Fix proposed by Peter McKeown
+			//
+			if ((this.sqlType == java.sql.Types.LONGVARBINARY) && !isBinary) {
+				this.sqlType = java.sql.Types.LONGVARCHAR;
+			} else if ((this.sqlType == java.sql.Types.VARBINARY) && !isBinary) {
+				this.sqlType = java.sql.Types.VARCHAR;
+			}
+		} else {
+			this.charsetName = "US-ASCII";
 		}
 
 		//
@@ -271,8 +279,6 @@ public class Field {
 				break;
 			}
 		}
-
-		checkForImplicitTemporaryTable();
 	}
 
 	/**
@@ -298,9 +304,6 @@ public class Field {
 		this.colDecimals = 0;
 	}
 
-	// ~ Methods
-	// ----------------------------------------------------------------
-
 	private void checkForImplicitTemporaryTable() {
 		this.isImplicitTempTable = this.tableNameLength > 5
 				&& this.buffer[tableNameStart] == (byte) '#'
@@ -312,10 +315,10 @@ public class Field {
 
 	/**
 	 * Returns the character set (if known) for this field.
-	 * 
+	 *
 	 * @return the character set
 	 */
-	public String getCharacterSet() {
+	public String getCharacterSet() throws SQLException {
 		return this.charsetName;
 	}
 
@@ -323,70 +326,71 @@ public class Field {
 		if (this.collationName == null) {
 			if (this.connection != null) {
 				if (this.connection.versionMeetsMinimum(4, 1, 0)) {
-					java.sql.DatabaseMetaData dbmd = this.connection
-							.getMetaData();
+					if (this.connection.getUseDynamicCharsetInfo()) {
+						java.sql.DatabaseMetaData dbmd = this.connection
+								.getMetaData();
 
-					String quotedIdStr = dbmd.getIdentifierQuoteString();
+						String quotedIdStr = dbmd.getIdentifierQuoteString();
 
-					if (" ".equals(quotedIdStr)) { //$NON-NLS-1$
-						quotedIdStr = ""; //$NON-NLS-1$
-					}
-
-					String csCatalogName = getDatabaseName();
-					String csTableName = getOriginalTableName();
-					String csColumnName = getOriginalName();
-
-					if (csCatalogName != null && csCatalogName.length() != 0
-							&& csTableName != null && csTableName.length() != 0
-							&& csColumnName != null
-							&& csColumnName.length() != 0) {
-						StringBuffer queryBuf = new StringBuffer(csCatalogName
-								.length()
-								+ csTableName.length() + 28);
-						queryBuf.append("SHOW FULL COLUMNS FROM "); //$NON-NLS-1$
-						queryBuf.append(quotedIdStr);
-						queryBuf.append(csCatalogName);
-						queryBuf.append(quotedIdStr);
-						queryBuf.append("."); //$NON-NLS-1$
-						queryBuf.append(quotedIdStr);
-						queryBuf.append(csTableName);
-						queryBuf.append(quotedIdStr);
-
-						java.sql.Statement collationStmt = null;
-						java.sql.ResultSet collationRs = null;
-
-						try {
-							collationStmt = this.connection.createStatement();
-
-							collationRs = collationStmt.executeQuery(queryBuf
-									.toString());
-
-							while (collationRs.next()) {
-								if (csColumnName.equals(collationRs
-										.getString("Field"))) { //$NON-NLS-1$
-									this.collationName = collationRs
-											.getString("Collation"); //$NON-NLS-1$
-
-									break;
-								}
-							}
-						} finally {
-							if (collationRs != null) {
-								collationRs.close();
-								collationRs = null;
-							}
-
-							if (collationStmt != null) {
-								collationStmt.close();
-								collationStmt = null;
-							}
+						if (" ".equals(quotedIdStr)) { //$NON-NLS-1$
+							quotedIdStr = ""; //$NON-NLS-1$
 						}
 
+						String csCatalogName = getDatabaseName();
+						String csTableName = getOriginalTableName();
+						String csColumnName = getOriginalName();
+
+						if (csCatalogName != null && csCatalogName.length() != 0
+								&& csTableName != null && csTableName.length() != 0
+								&& csColumnName != null
+								&& csColumnName.length() != 0) {
+							StringBuffer queryBuf = new StringBuffer(csCatalogName
+									.length()
+									+ csTableName.length() + 28);
+							queryBuf.append("SHOW FULL COLUMNS FROM "); //$NON-NLS-1$
+							queryBuf.append(quotedIdStr);
+							queryBuf.append(csCatalogName);
+							queryBuf.append(quotedIdStr);
+							queryBuf.append("."); //$NON-NLS-1$
+							queryBuf.append(quotedIdStr);
+							queryBuf.append(csTableName);
+							queryBuf.append(quotedIdStr);
+
+							java.sql.Statement collationStmt = null;
+							java.sql.ResultSet collationRs = null;
+
+							try {
+								collationStmt = this.connection.createStatement();
+
+								collationRs = collationStmt.executeQuery(queryBuf
+										.toString());
+
+								while (collationRs.next()) {
+									if (csColumnName.equals(collationRs
+											.getString("Field"))) { //$NON-NLS-1$
+										this.collationName = collationRs
+												.getString("Collation"); //$NON-NLS-1$
+
+										break;
+									}
+								}
+							} finally {
+								if (collationRs != null) {
+									collationRs.close();
+									collationRs = null;
+								}
+
+								if (collationStmt != null) {
+									collationStmt.close();
+									collationStmt = null;
+								}
+							}
+						}
+					} else {
+						this.collationName = CharsetMapping.INDEX_TO_COLLATION[charsetIndex];
 					}
 				}
-
 			}
-
 		}
 
 		return this.collationName;
@@ -398,7 +402,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getDatabaseName() throws SQLException {
@@ -417,7 +421,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getFullName() throws SQLException {
@@ -438,7 +442,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getFullOriginalName() throws SQLException {
@@ -466,20 +470,24 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public long getLength() {
 		return this.length;
 	}
 
-	public int getMaxBytesPerCharacter() throws SQLException {
-		return this.connection.getMaxBytesPerChar(getCharacterSet());
+	public synchronized int getMaxBytesPerCharacter() throws SQLException {
+		if (this.maxBytesPerChar == 0) {
+			this.maxBytesPerChar = this.connection.getMaxBytesPerChar(getCharacterSet());
+		}
+
+		return this.maxBytesPerChar;
 	}
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public int getMysqlType() {
@@ -488,7 +496,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getName() throws SQLException {
@@ -503,18 +511,18 @@ public class Field {
 		if (this.useOldNameMetadata) {
 			return getName();
 		}
-		
-		if (this.connection != null && 
+
+		if (this.connection != null &&
 				this.connection.versionMeetsMinimum(4, 1, 0)) {
 			return getOriginalName();
 		}
-		
+
 		return getName();
 	}
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getOriginalName() throws SQLException {
@@ -530,7 +538,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getOriginalTableName() throws SQLException {
@@ -547,9 +555,9 @@ public class Field {
 	/**
 	 * Returns amount of correction that should be applied to the precision
 	 * value.
-	 * 
+	 *
 	 * Different versions of MySQL report different precision values.
-	 * 
+	 *
 	 * @return the amount to adjust precision value by.
 	 */
 	public int getPrecisionAdjustFactor() {
@@ -558,7 +566,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public int getSQLType() {
@@ -636,7 +644,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getTable() throws SQLException {
@@ -645,7 +653,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public String getTableName() throws SQLException {
@@ -661,13 +669,13 @@ public class Field {
 		if (this.connection.versionMeetsMinimum(4, 1, 0)) {
 			return getOriginalTableName();
 		}
-			
+
 		return getTableName(); // pre-4.1, no aliases returned
 	}
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isAutoIncrement() {
@@ -676,7 +684,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isBinary() {
@@ -685,7 +693,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isBlob() {
@@ -694,7 +702,7 @@ public class Field {
 
 	/**
 	 * Is this field owned by a server-created temporary table?
-	 * 
+	 *
 	 * @return
 	 */
 	private boolean isImplicitTemporaryTable() {
@@ -703,7 +711,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isMultipleKey() {
@@ -725,7 +733,8 @@ public class Field {
 				&& (this.getMysqlType() == MysqlDefs.FIELD_TYPE_STRING ||
 				this.getMysqlType() == MysqlDefs.FIELD_TYPE_VAR_STRING)) {
 
-			if (this.originalTableNameLength == 0) {
+			if (this.originalTableNameLength == 0 && (
+					this.connection != null && !this.connection.versionMeetsMinimum(5, 0, 25))) {
 				return false; // Probably from function
 			}
 
@@ -742,7 +751,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isPrimaryKey() {
@@ -751,7 +760,7 @@ public class Field {
 
 	/**
 	 * Is this field _definitely_ not writable?
-	 * 
+	 *
 	 * @return true if this field can not be written to in an INSERT/UPDATE
 	 *         statement.
 	 */
@@ -769,7 +778,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isUniqueKey() {
@@ -778,7 +787,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isUnsigned() {
@@ -787,7 +796,7 @@ public class Field {
 
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @return DOCUMENT ME!
 	 */
 	public boolean isZeroFill() {
@@ -811,9 +820,24 @@ public class Field {
 		}
 	}
 
+	private boolean isNativeNumericType() {
+		return ((this.mysqlType >= MysqlDefs.FIELD_TYPE_TINY &&
+					this.mysqlType <= MysqlDefs.FIELD_TYPE_DOUBLE) ||
+					this.mysqlType == MysqlDefs.FIELD_TYPE_LONGLONG ||
+					this.mysqlType == MysqlDefs.FIELD_TYPE_YEAR);
+	}
+
+	private boolean isNativeDateTimeType() {
+		return (this.mysqlType == MysqlDefs.FIELD_TYPE_DATE ||
+				this.mysqlType == MysqlDefs.FIELD_TYPE_NEWDATE ||
+				this.mysqlType == MysqlDefs.FIELD_TYPE_DATETIME ||
+				this.mysqlType == MysqlDefs.FIELD_TYPE_TIME ||
+				this.mysqlType == MysqlDefs.FIELD_TYPE_TIMESTAMP);
+	}
+
 	/**
 	 * DOCUMENT ME!
-	 * 
+	 *
 	 * @param conn
 	 *            DOCUMENT ME!
 	 */
@@ -834,7 +858,7 @@ public class Field {
 
 	public String toString() {
 		try {
-			StringBuffer asString = new StringBuffer();
+			StringBuffer asString = new StringBuffer(128);
 			asString.append(super.toString());
 
 			asString.append("\n  catalog: ");
@@ -849,6 +873,9 @@ public class Field {
 			asString.append(this.getOriginalName());
 			asString.append("\n  MySQL data type: ");
 			asString.append(getMysqlType());
+			asString.append("(");
+			asString.append(MysqlDefs.typeToName(getMysqlType()));
+			asString.append(")");
 
 			if (this.buffer != null) {
 				asString.append("\n\nData as received from server:\n\n");
