@@ -39,7 +39,7 @@ import java.util.Properties;
  * @version $Id: ReplicationConnection.java,v 1.1.2.1 2005/05/13 18:58:38
  *          mmatthews Exp $
  */
-public class ReplicationConnection implements java.sql.Connection {
+public class ReplicationConnection implements java.sql.Connection, PingTarget {
 	private Connection currentConnection;
 
 	private Connection masterConnection;
@@ -127,7 +127,10 @@ public class ReplicationConnection implements java.sql.Connection {
 	 * @see java.sql.Connection#createStatement()
 	 */
 	public Statement createStatement() throws SQLException {
-		return this.currentConnection.createStatement();
+		Statement stmt = this.currentConnection.createStatement();
+		((com.mysql.jdbc.Statement) stmt).setPingTarget(this);
+		
+		return stmt;
 	}
 
 	/*
@@ -137,8 +140,12 @@ public class ReplicationConnection implements java.sql.Connection {
 	 */
 	public synchronized Statement createStatement(int resultSetType,
 			int resultSetConcurrency) throws SQLException {
-		return this.currentConnection.createStatement(resultSetType,
+		Statement stmt = this.currentConnection.createStatement(resultSetType,
 				resultSetConcurrency);
+		
+		((com.mysql.jdbc.Statement) stmt).setPingTarget(this);
+		
+		return stmt;
 	}
 
 	/*
@@ -149,8 +156,12 @@ public class ReplicationConnection implements java.sql.Connection {
 	public synchronized Statement createStatement(int resultSetType,
 			int resultSetConcurrency, int resultSetHoldability)
 			throws SQLException {
-		return this.currentConnection.createStatement(resultSetType,
+		Statement stmt = this.currentConnection.createStatement(resultSetType,
 				resultSetConcurrency, resultSetHoldability);
+		
+		((com.mysql.jdbc.Statement) stmt).setPingTarget(this);
+		
+		return stmt;
 	}
 
 	/*
@@ -293,7 +304,11 @@ public class ReplicationConnection implements java.sql.Connection {
 	 * @see java.sql.Connection#prepareStatement(java.lang.String)
 	 */
 	public PreparedStatement prepareStatement(String sql) throws SQLException {
-		return this.currentConnection.prepareStatement(sql);
+		PreparedStatement pstmt = this.currentConnection.prepareStatement(sql);
+
+		((com.mysql.jdbc.Statement) pstmt).setPingTarget(this);
+		
+		return pstmt;
 	}
 
 	/*
@@ -303,7 +318,11 @@ public class ReplicationConnection implements java.sql.Connection {
 	 */
 	public synchronized PreparedStatement prepareStatement(String sql,
 			int autoGeneratedKeys) throws SQLException {
-		return this.currentConnection.prepareStatement(sql, autoGeneratedKeys);
+		PreparedStatement pstmt = this.currentConnection.prepareStatement(sql, autoGeneratedKeys);
+
+		((com.mysql.jdbc.Statement) pstmt).setPingTarget(this);
+		
+		return pstmt;
 	}
 
 	/*
@@ -313,8 +332,12 @@ public class ReplicationConnection implements java.sql.Connection {
 	 */
 	public synchronized PreparedStatement prepareStatement(String sql,
 			int resultSetType, int resultSetConcurrency) throws SQLException {
-		return this.currentConnection.prepareStatement(sql, resultSetType,
+		PreparedStatement pstmt = this.currentConnection.prepareStatement(sql, resultSetType,
 				resultSetConcurrency);
+		
+		((com.mysql.jdbc.Statement) pstmt).setPingTarget(this);
+		
+		return pstmt;
 	}
 
 	/*
@@ -326,8 +349,12 @@ public class ReplicationConnection implements java.sql.Connection {
 	public synchronized PreparedStatement prepareStatement(String sql,
 			int resultSetType, int resultSetConcurrency,
 			int resultSetHoldability) throws SQLException {
-		return this.currentConnection.prepareStatement(sql, resultSetType,
+		PreparedStatement pstmt = this.currentConnection.prepareStatement(sql, resultSetType,
 				resultSetConcurrency, resultSetHoldability);
+
+		((com.mysql.jdbc.Statement) pstmt).setPingTarget(this);
+		
+		return pstmt;
 	}
 
 	/*
@@ -337,7 +364,11 @@ public class ReplicationConnection implements java.sql.Connection {
 	 */
 	public synchronized PreparedStatement prepareStatement(String sql,
 			int[] columnIndexes) throws SQLException {
-		return this.currentConnection.prepareStatement(sql, columnIndexes);
+		PreparedStatement pstmt = this.currentConnection.prepareStatement(sql, columnIndexes);
+		
+		((com.mysql.jdbc.Statement) pstmt).setPingTarget(this);
+		
+		return pstmt;
 	}
 
 	/*
@@ -348,7 +379,11 @@ public class ReplicationConnection implements java.sql.Connection {
 	 */
 	public synchronized PreparedStatement prepareStatement(String sql,
 			String[] columnNames) throws SQLException {
-		return this.currentConnection.prepareStatement(sql, columnNames);
+		PreparedStatement pstmt = this.currentConnection.prepareStatement(sql, columnNames);
+
+		((com.mysql.jdbc.Statement) pstmt).setPingTarget(this);
+		
+		return pstmt;
 	}
 
 	/*
@@ -464,11 +499,11 @@ public class ReplicationConnection implements java.sql.Connection {
 		this.currentConnection.setTypeMap(arg0);
 	}
 
-	private synchronized void switchToSlavesConnection() throws SQLException {
+	private synchronized void switchToMasterConnection() throws SQLException {
 		swapConnections(this.masterConnection, this.slavesConnection);
 	}
 
-	private synchronized void switchToMasterConnection() throws SQLException {
+	private synchronized void switchToSlavesConnection() throws SQLException {
 		swapConnections(this.slavesConnection, this.masterConnection);
 	}
 	
@@ -477,36 +512,49 @@ public class ReplicationConnection implements java.sql.Connection {
 	 * sourceConnection to targetConnection, and makes targetConnection
 	 * the "current" connection that will be used for queries.
 	 * 
-	 * @param sourceConnection the connection to swap from
-	 * @param targetConnection the connection to swap to
+	 * @param switchToConnection the connection to swap from
+	 * @param switchFromConnection the connection to swap to
 	 * 
 	 * @throws SQLException if an error occurs
 	 */
-	private synchronized void swapConnections(Connection sourceConnection, 
-			Connection targetConnection) throws SQLException {
-		String targetCatalog = targetConnection.getCatalog();
-		String sourceCatalog = sourceConnection.getCatalog();
+	private synchronized void swapConnections(Connection switchToConnection, 
+			Connection switchFromConnection) throws SQLException {
+		String switchFromCatalog = switchFromConnection.getCatalog();
+		String switchToCatalog = switchToConnection.getCatalog();
 
-		if (sourceCatalog != null && !sourceCatalog.equals(targetCatalog)) {
-			targetConnection.setCatalog(targetCatalog);
-		} else if (targetCatalog != null) {
-			targetConnection.setCatalog(targetCatalog);
+		if (switchToCatalog != null && !switchToCatalog.equals(switchFromCatalog)) {
+			switchToConnection.setCatalog(switchFromCatalog);
+		} else if (switchFromCatalog != null) {
+			switchToConnection.setCatalog(switchFromCatalog);
 		}
 
-		boolean sourceAutoCommit = sourceConnection.getAutoCommit();
-
-		if (targetConnection.getAutoCommit() != sourceAutoCommit) {
-			targetConnection.setAutoCommit(sourceAutoCommit);
+		boolean switchToAutoCommit = switchToConnection.getAutoCommit();
+		boolean switchFromConnectionAutoCommit = switchFromConnection.getAutoCommit();
+		
+		if (switchFromConnectionAutoCommit != switchToAutoCommit) {
+			switchToConnection.setAutoCommit(switchFromConnectionAutoCommit);
 		}
 
-		int sourceTransactionIsolation = sourceConnection
+		int switchToIsolation = switchToConnection
 				.getTransactionIsolation();
 
-		if (targetConnection.getTransactionIsolation() != sourceTransactionIsolation) {
-			targetConnection
-					.setTransactionIsolation(sourceTransactionIsolation);
+		int switchFromIsolation = switchFromConnection.getTransactionIsolation();
+		
+		if (switchFromIsolation != switchToIsolation) {
+			switchToConnection
+					.setTransactionIsolation(switchFromIsolation);
 		}
 		
-		this.currentConnection = targetConnection;
+		this.currentConnection = switchToConnection;
 	}
-}
+
+	public synchronized void doPing() throws SQLException {
+		if (this.masterConnection != null) {
+			this.masterConnection.ping();
+		}
+		
+		if (this.slavesConnection != null) {
+			this.slavesConnection.ping();
+		}
+	}
+}
\ No newline at end of file
