@@ -24,6 +24,9 @@
  */
 package com.mysql.jdbc;
 
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.io.ObjectInputStream;
 import java.sql.SQLException;
 import java.sql.Types;
 
@@ -75,15 +78,17 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 	}
 
 	Field[] fields;
-
+	boolean useOldAliasBehavior = false;
+	
 	/**
 	 * Initialise for a result with a tuple set and a field descriptor set
 	 * 
 	 * @param fields
 	 *            the array of field descriptors
 	 */
-	public ResultSetMetaData(Field[] fields) {
+	public ResultSetMetaData(Field[] fields, boolean useOldAliasBehavior) {
 		this.fields = fields;
+		this.useOldAliasBehavior = useOldAliasBehavior;
 	}
 
 	/**
@@ -148,6 +153,36 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 
 	// --------------------------JDBC 2.0-----------------------------------
 
+	/**
+	 * JDBC 2.0
+	 * 
+	 * <p>
+	 * Return the fully qualified name of the Java class whose instances are
+	 * manufactured if ResultSet.getObject() is called to retrieve a value from
+	 * the column. ResultSet.getObject() may return a subClass of the class
+	 * returned by this method.
+	 * </p>
+	 * 
+	 * @param column
+	 *            the column number to retrieve information for
+	 * 
+	 * @return the fully qualified name of the Java class whose instances are
+	 *         manufactured if ResultSet.getObject() is called to retrieve a
+	 *         value from the column.
+	 * 
+	 * @throws SQLException
+	 *             if an error occurs
+	 */
+	public String getColumnClassName(int column) throws SQLException {
+		Field f = getField(column);
+
+		return getClassNameForJavaType(f.getSQLType(), 
+				f.isUnsigned(), 
+				f.getMysqlType(), 
+				f.isBinary() || f.isBlob(),
+				f.isOpaqueBinary()); 
+	}
+
 	/**
 	 * Whats the number of columns in the ResultSet?
 	 * 
@@ -191,7 +226,11 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 	 *             if a database access error occurs
 	 */
 	public String getColumnLabel(int column) throws SQLException {
-		return getColumnName(column);
+		if (this.useOldAliasBehavior) {
+			return getColumnName(column);
+		}
+		
+		return getField(column).getColumnLabel();
 	}
 
 	/**
@@ -206,7 +245,17 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 	 *             if a databvase access error occurs
 	 */
 	public String getColumnName(int column) throws SQLException {
-		return getField(column).getName();
+		if (this.useOldAliasBehavior) {
+			return getField(column).getName();
+		}
+		
+		String name = getField(column).getNameNoAliases();
+		
+		if (name != null && name.length() == 0) {
+			return getField(column).getName();
+		}
+		
+		return name;
 	}
 
 	/**
@@ -241,6 +290,7 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 		Field field = getField(column);
 
 		int mysqlType = field.getMysqlType();
+		int jdbcType = field.getSQLType();
 
 		switch (mysqlType) {
 		case MysqlDefs.FIELD_TYPE_BIT:
@@ -256,7 +306,7 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 			return field.isUnsigned() ? "SMALLINT UNSIGNED" : "SMALLINT";
 
 		case MysqlDefs.FIELD_TYPE_LONG:
-			return field.isUnsigned() ? "INTEGER UNSIGNED" : "INTEGER";
+			return field.isUnsigned() ? "INT UNSIGNED" : "INT";
 
 		case MysqlDefs.FIELD_TYPE_FLOAT:
 			return field.isUnsigned() ? "FLOAT UNSIGNED" : "FLOAT";
@@ -305,9 +355,17 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 			return "VARCHAR"; //$NON-NLS-1$
 
 		case MysqlDefs.FIELD_TYPE_VAR_STRING:
+			if (jdbcType == Types.VARBINARY) {
+				return "VARBINARY";
+			}
+			
 			return "VARCHAR"; //$NON-NLS-1$
 
 		case MysqlDefs.FIELD_TYPE_STRING:
+			if (jdbcType == Types.BINARY) {
+				return "BINARY";
+			}
+			
 			return "CHAR"; //$NON-NLS-1$
 
 		case MysqlDefs.FIELD_TYPE_ENUM:
@@ -337,7 +395,7 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 	 */
 	protected Field getField(int columnIndex) throws SQLException {
 		if ((columnIndex < 1) || (columnIndex > this.fields.length)) {
-			throw new SQLException(Messages.getString("ResultSetMetaData.46"), //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("ResultSetMetaData.46"), //$NON-NLS-1$
 					SQLError.SQL_STATE_INVALID_COLUMN_NUMBER);
 		}
 
@@ -437,7 +495,11 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 	 *             if a database access error occurs
 	 */
 	public String getTableName(int column) throws SQLException {
-		return getField(column).getTableName();
+		if (this.useOldAliasBehavior) {
+			return getField(column).getTableName();
+		}
+		
+		return getField(column).getTableNameNoAliases();
 	}
 
 	/**
@@ -624,12 +686,6 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 		}
 	}
 
-	// *********************************************************************
-	//
-	// END OF PUBLIC INTERFACE
-	//
-	// *********************************************************************
-
 	/**
 	 * Is it possible for a write on the column to succeed?
 	 * 
@@ -662,7 +718,7 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 
 		return toStringBuf.toString();
 	}
-
+	
 	static String getClassNameForJavaType(int javaType, 
 			boolean isUnsigned, int mysqlTypeIfKnown, 
 			boolean isBinaryOrBlob,
@@ -671,64 +727,64 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 		case Types.BIT:
 		case Types.BOOLEAN:
 			return "java.lang.Boolean"; //$NON-NLS-1$
-	
+
 		case Types.TINYINT:
-	
+
 			if (isUnsigned) {
 				return "java.lang.Integer"; //$NON-NLS-1$
 			}
-	
+
 			return "java.lang.Integer"; //$NON-NLS-1$
-	
+
 		case Types.SMALLINT:
-	
+
 			if (isUnsigned) {
 				return "java.lang.Integer"; //$NON-NLS-1$
 			}
-	
+
 			return "java.lang.Integer"; //$NON-NLS-1$
-	
+
 		case Types.INTEGER:
-	
+
 			if (!isUnsigned || 
 					mysqlTypeIfKnown == MysqlDefs.FIELD_TYPE_INT24) {
 				return "java.lang.Integer"; //$NON-NLS-1$
 			}
-	
+
 			return "java.lang.Long"; //$NON-NLS-1$
-	
+
 		case Types.BIGINT:
-	
+
 			if (!isUnsigned) {
 				return "java.lang.Long"; //$NON-NLS-1$
 			}
-	
+
 			return "java.math.BigInteger"; //$NON-NLS-1$
-	
+
 		case Types.DECIMAL:
 		case Types.NUMERIC:
 			return "java.math.BigDecimal"; //$NON-NLS-1$
-	
+
 		case Types.REAL:
 			return "java.lang.Float"; //$NON-NLS-1$
-	
+
 		case Types.FLOAT:
 		case Types.DOUBLE:
 			return "java.lang.Double"; //$NON-NLS-1$
-	
+
 		case Types.CHAR:
 		case Types.VARCHAR:
 		case Types.LONGVARCHAR:
 			if (!isOpaqueBinary) {
 				return "java.lang.String"; //$NON-NLS-1$
 			}
-	
+
 			return "[B";
-	
+
 		case Types.BINARY:
 		case Types.VARBINARY:
 		case Types.LONGVARBINARY:
-	
+
 			if (mysqlTypeIfKnown == MysqlDefs.FIELD_TYPE_GEOMETRY) {
 				return "[B";
 			} else if (isBinaryOrBlob) {
@@ -736,50 +792,18 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 			} else {
 				return "java.lang.String";
 			}
-	
+
 		case Types.DATE:
 			return "java.sql.Date"; //$NON-NLS-1$
-	
+
 		case Types.TIME:
 			return "java.sql.Time"; //$NON-NLS-1$
-	
+
 		case Types.TIMESTAMP:
 			return "java.sql.Timestamp"; //$NON-NLS-1$
-	
+
 		default:
 			return "java.lang.Object"; //$NON-NLS-1$
 		}
 	}
-
-	// --------------------------JDBC 2.0-----------------------------------
-	
-	/**
-	 * JDBC 2.0
-	 * 
-	 * <p>
-	 * Return the fully qualified name of the Java class whose instances are
-	 * manufactured if ResultSet.getObject() is called to retrieve a value from
-	 * the column. ResultSet.getObject() may return a subClass of the class
-	 * returned by this method.
-	 * </p>
-	 * 
-	 * @param column
-	 *            the column number to retrieve information for
-	 * 
-	 * @return the fully qualified name of the Java class whose instances are
-	 *         manufactured if ResultSet.getObject() is called to retrieve a
-	 *         value from the column.
-	 * 
-	 * @throws SQLException
-	 *             if an error occurs
-	 */
-	public String getColumnClassName(int column) throws SQLException {
-		Field f = getField(column);
-	
-		return getClassNameForJavaType(f.getSQLType(), 
-				f.isUnsigned(), 
-				f.getMysqlType(), 
-				f.isBinary() || f.isBlob(),
-				f.isOpaqueBinary()); 
-	}
 }
