@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2002-2006 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -33,7 +33,7 @@ import com.mysql.jdbc.profiler.ProfilerEvent;
  * Allows streaming of MySQL data.
  * 
  * @author dgan
- * @version $Id: RowDataDynamic.java 4891 2006-02-03 19:10:02Z mmatthews $
+ * @version $Id: RowDataDynamic.java 6433 2007-05-18 18:38:56Z mmatthews $
  */
 public class RowDataDynamic implements RowData {
 	// ~ Instance fields
@@ -65,6 +65,8 @@ public class RowDataDynamic implements RowData {
 	private ResultSet owner;
 
 	private boolean streamerClosed = false;
+	
+	private boolean wasEmpty = false; // we don't know until we attempt to traverse
 
 	// ~ Methods
 	// ----------------------------------------------------------------
@@ -177,6 +179,7 @@ public class RowDataDynamic implements RowData {
 									-1,
 									System.currentTimeMillis(),
 									0,
+									Constants.MILLIS_I18N,
 									null,
 									null,
 									Messages.getString("RowDataDynamic.2") //$NON-NLS-1$
@@ -354,6 +357,7 @@ public class RowDataDynamic implements RowData {
 		return ret;
 	}
 
+
 	private void nextRecord() throws SQLException {
 
 		try {
@@ -365,10 +369,19 @@ public class RowDataDynamic implements RowData {
 
 				if (this.nextRow == null) {
 					this.isAtEnd = true;
+					
+					if (this.index == -1) {
+						this.wasEmpty = true;
+					}
 				}
 			} else {
 				this.isAfterEnd = true;
 			}
+		} catch (CommunicationsException comEx) {
+			// Give a better error message
+			comEx.setWasStreamingResults();
+			
+			throw comEx;
 		} catch (SQLException sqlEx) {
 			// don't wrap SQLExceptions
 			throw sqlEx;
@@ -433,4 +446,10 @@ public class RowDataDynamic implements RowData {
 		return RESULT_SET_SIZE_UNKNOWN;
 	}
 
+	public boolean wasEmpty() {
+		return this.wasEmpty;
+	}
+	
+	
+
 }
