@@ -34,6 +34,12 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.TreeMap;
 
+import com.mysql.jdbc.exceptions.MySQLDataException;
+import com.mysql.jdbc.exceptions.MySQLIntegrityConstraintViolationException;
+import com.mysql.jdbc.exceptions.MySQLNonTransientConnectionException;
+import com.mysql.jdbc.exceptions.MySQLSyntaxErrorException;
+import com.mysql.jdbc.exceptions.MySQLTransactionRollbackException;
+
 /**
  * SQLError is a utility class that maps MySQL error codes to X/Open error codes
  * as is required by the JDBC spec.
@@ -121,6 +127,8 @@ public class SQLError {
 	public static final String SQL_STATE_UNABLE_TO_CONNECT_TO_DATASOURCE = "08001"; //$NON-NLS-1$
 
 	public static final String SQL_STATE_WRONG_NO_OF_PARAMETERS = "07001"; //$NON-NLS-1$
+	
+	public static final String SQL_STATE_INVALID_TRANSACTION_TERMINATION = "2D000"; //$NON_NLS-1$
 
 	private static Map sqlStateMessages;
 
@@ -861,4 +869,80 @@ public class SQLError {
 
 		return SQL_STATE_GENERAL_ERROR;
 	}
+
+	/*
+	 * SQL State Class SQLNonTransientException Subclass 08
+	 * SQLNonTransientConnectionException 22 SQLDataException 23
+	 * SQLIntegrityConstraintViolationException N/A
+	 * SQLInvalidAuthorizationException 42 SQLSyntaxErrorException
+	 * 
+	 * SQL State Class SQLTransientException Subclass 08
+	 * SQLTransientConnectionException 40 SQLTransactionRollbackException N/A
+	 * SQLTimeoutException
+	 */
+
+	public static SQLException createSQLException(String message,
+			String sqlState) {
+		if (sqlState != null) {
+			if (sqlState.startsWith("08")) {
+				return new MySQLNonTransientConnectionException(message,
+						sqlState);
+			}
+
+			if (sqlState.startsWith("22")) {
+				return new MySQLDataException(message, sqlState);
+			}
+
+			if (sqlState.startsWith("23")) {
+				return new MySQLIntegrityConstraintViolationException(message,
+						sqlState);
+			}
+
+			if (sqlState.startsWith("42")) {
+				return new MySQLSyntaxErrorException(message, sqlState);
+			}
+
+			if (sqlState.startsWith("40")) {
+				return new MySQLTransactionRollbackException(message, sqlState);
+			}
+		}
+
+		return new SQLException(message, sqlState);
+	}
+
+	public static SQLException createSQLException(String message) {
+		return new SQLException(message);
+	}
+
+	public static SQLException createSQLException(String message,
+			String sqlState, int vendorErrorCode) {
+		if (sqlState != null) {
+			if (sqlState.startsWith("08")) {
+				return new MySQLNonTransientConnectionException(message,
+						sqlState, vendorErrorCode);
+			}
+
+			if (sqlState.startsWith("22")) {
+				return new MySQLDataException(message, sqlState,
+						vendorErrorCode);
+			}
+
+			if (sqlState.startsWith("23")) {
+				return new MySQLIntegrityConstraintViolationException(message,
+						sqlState, vendorErrorCode);
+			}
+
+			if (sqlState.startsWith("42")) {
+				return new MySQLSyntaxErrorException(message, sqlState,
+						vendorErrorCode);
+			}
+
+			if (sqlState.startsWith("40")) {
+				return new MySQLTransactionRollbackException(message, sqlState,
+						vendorErrorCode);
+			}
+		}
+
+		return new SQLException(message, sqlState, vendorErrorCode);
+	}
 }
