@@ -25,6 +25,8 @@
 package com.mysql.jdbc;
 
 import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.StringReader;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
@@ -314,13 +316,17 @@ public class StringUtils {
 	 * @return the first non-whitespace character, upper cased.
 	 */
 	public static char firstNonWsCharUc(String searchIn) {
+		return firstNonWsCharUc(searchIn, 0);
+	}
+	
+	public static char firstNonWsCharUc(String searchIn, int startAt) {
 		if (searchIn == null) {
 			return 0;
 		}
 
 		int length = searchIn.length();
 
-		for (int i = 0; i < length; i++) {
+		for (int i = startAt; i < length; i++) {
 			char c = searchIn.charAt(i);
 
 			if (!Character.isWhitespace(c)) {
@@ -620,17 +626,17 @@ public class StringUtils {
 		}
 	}
 
-	public static int getInt(byte[] buf) throws NumberFormatException {
+	public static int getInt(byte[] buf, int offset, int endPos) throws NumberFormatException {
 		int base = 10;
 
-		int s = 0;
+		int s = offset;
 
 		/* Skip white space. */
-		while (Character.isWhitespace((char) buf[s]) && (s < buf.length)) {
+		while (Character.isWhitespace((char) buf[s]) && (s < endPos)) {
 			++s;
 		}
 
-		if (s == buf.length) {
+		if (s == endPos) {
 			throw new NumberFormatException(new String(buf));
 		}
 
@@ -658,7 +664,7 @@ public class StringUtils {
 
 		int i = 0;
 
-		for (; s < buf.length; s++) {
+		for (; s < endPos; s++) {
 			char c = (char) buf[s];
 
 			if (Character.isDigit(c)) {
@@ -693,6 +699,10 @@ public class StringUtils {
 		/* Return the result of the appropriate sign. */
 		return (negative ? (-i) : i);
 	}
+	
+	public static int getInt(byte[] buf) throws NumberFormatException {
+		return getInt(buf, 0, buf.length);
+	}
 
 	public static long getLong(byte[] buf) throws NumberFormatException {
 		int base = 10;
@@ -925,7 +935,7 @@ public class StringUtils {
 			boolean allowBackslashEscapes) {
 		char contextMarker = Character.MIN_VALUE;
 		boolean escaped = false;
-		int markerTypeFound = 0;
+		int markerTypeFound = -1;
 		int srcLength = src.length();
 		int ind = 0;
 
@@ -934,8 +944,9 @@ public class StringUtils {
 
 			if (allowBackslashEscapes && c == '\\') {
 				escaped = !escaped;
-			} else if (c == markerCloses.charAt(markerTypeFound) && !escaped) {
+			} else if (markerTypeFound != -1 && c == markerCloses.charAt(markerTypeFound) && !escaped) {
 				contextMarker = Character.MIN_VALUE;
+				markerTypeFound = -1;
 			} else if ((ind = marker.indexOf(c)) != -1 && !escaped
 					&& contextMarker == Character.MIN_VALUE) {
 				markerTypeFound = ind;
@@ -968,7 +979,10 @@ public class StringUtils {
 			} else if (c == quoteChar && !escaped
 					&& contextMarker == Character.MIN_VALUE) {
 				contextMarker = c;
-			} else if (c == target.charAt(0) && !escaped
+			// This test looks complex, but remember that in certain locales, upper case
+			// of two different codepoints coverts to same codepoint, and vice-versa.
+			} else if ((Character.toUpperCase(c) == Character.toUpperCase(target.charAt(0)) ||
+					Character.toLowerCase(c) == Character.toLowerCase(target.charAt(0))) && !escaped
 					&& contextMarker == Character.MIN_VALUE) {
 				if (startsWithIgnoreCase(src, i, target))
 					return i;
@@ -1445,4 +1459,163 @@ public class StringUtils {
 
 		return s.getBytes();
 	}
+
+	public static int lastIndexOf(byte[] s, char c) {
+		if (s == null) {
+			return -1;
+		}
+		
+		for (int i = s.length - 1; i >= 0; i--) {
+			if (s[i] == c) {
+				return i;
+			}
+		}
+		
+		return -1;
+	}
+
+	public static int indexOf(byte[] s, char c) {
+		if (s == null) {
+			return -1;
+		}
+		
+		int length = s.length;
+		
+		for (int i = 0; i < length; i++) {
+			if (s[i] == c) {
+				return i;
+			}
+		}
+		
+		return -1;
+	}
+	
+	/**
+	 * Returns the given string, with comments removed
+	 * 
+	 * @param src
+	 *            the source string
+	 * @param stringOpens
+	 *            characters which delimit the "open" of a string
+	 * @param stringCloses
+	 *            characters which delimit the "close" of a string, in
+	 *            counterpart order to <code>stringOpens</code>
+	 * @param slashStarComments
+	 *            strip slash-star type "C" style comments
+	 * @param slashSlashComments
+	 *            strip slash-slash C++ style comments to end-of-line
+	 * @param hashComments
+	 *            strip #-style comments to end-of-line
+	 * @param dashDashComments
+	 *            strip "--" style comments to end-of-line
+	 * @return the input string with all comment-delimited data removed
+	 */
+	public static String stripComments(String src, String stringOpens,
+			String stringCloses, boolean slashStarComments,
+			boolean slashSlashComments, boolean hashComments,
+			boolean dashDashComments) {
+		if (src == null) {
+			return null;
+		}
+
+		StringBuffer buf = new StringBuffer(src.length());
+
+		// It's just more natural to deal with this as a stream
+		// when parsing..This code is currently only called when
+		// parsing the kind of metadata that developers are strongly
+		// recommended to cache anyways, so we're not worried
+		// about the _1_ extra object allocation if it cleans
+		// up the code
+
+		StringReader sourceReader = new StringReader(src);
+
+		int contextMarker = Character.MIN_VALUE;
+		boolean escaped = false;
+		int markerTypeFound = -1;
+
+		int ind = 0;
+
+		int currentChar = 0;
+
+		try {
+			while ((currentChar = sourceReader.read()) != -1) {
+
+				if (false && currentChar == '\\') {
+					escaped = !escaped;
+				} else if (markerTypeFound != -1 && currentChar == stringCloses.charAt(markerTypeFound)
+						&& !escaped) {
+					contextMarker = Character.MIN_VALUE;
+					markerTypeFound = -1;
+				} else if ((ind = stringOpens.indexOf(currentChar)) != -1
+						&& !escaped && contextMarker == Character.MIN_VALUE) {
+					markerTypeFound = ind;
+					contextMarker = currentChar;
+				}
+
+				if (contextMarker == Character.MIN_VALUE && currentChar == '/'
+						&& (slashSlashComments || slashStarComments)) {
+					currentChar = sourceReader.read();
+					if (currentChar == '*' && slashStarComments) {
+						int prevChar = 0;
+						while ((currentChar = sourceReader.read()) != '/'
+								|| prevChar != '*') {
+							if (currentChar == '\r') {
+
+								currentChar = sourceReader.read();
+								if (currentChar == '\n') {
+									currentChar = sourceReader.read();
+								}
+							} else {
+								if (currentChar == '\n') {
+
+									currentChar = sourceReader.read();
+								}
+							}
+							if (currentChar < 0)
+								break;
+							prevChar = currentChar;
+						}
+						continue;
+					} else if (currentChar == '/' && slashSlashComments) {
+						while ((currentChar = sourceReader.read()) != '\n'
+								&& currentChar != '\r' && currentChar >= 0)
+							;
+					}
+				} else if (contextMarker == Character.MIN_VALUE
+						&& currentChar == '#' && hashComments) {
+					// Slurp up everything until the newline
+					while ((currentChar = sourceReader.read()) != '\n'
+							&& currentChar != '\r' && currentChar >= 0)
+						;
+				} else if (contextMarker == Character.MIN_VALUE
+						&& currentChar == '-' && dashDashComments) {
+					currentChar = sourceReader.read();
+
+					if (currentChar == -1 || currentChar != '-') {
+						buf.append('-');
+
+						if (currentChar != -1) {
+							buf.append(currentChar);
+						}
+
+						continue;
+					}
+
+					// Slurp up everything until the newline
+
+					while ((currentChar = sourceReader.read()) != '\n'
+							&& currentChar != '\r' && currentChar >= 0)
+						;
+				}
+
+				if (currentChar != -1) {
+					buf.append((char) currentChar);
+				}
+			}
+		} catch (IOException ioEx) {
+			// we'll never see this from a StringReader
+		}
+
+		return buf.toString();
+	}
 }
