@@ -31,6 +31,7 @@ import java.sql.Timestamp;
 
 import java.util.Calendar;
 import java.util.Collections;
+import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
@@ -808,34 +809,51 @@ public class TimeUtil {
 	 * 
 	 * @return the times changed to the timezone 'toTz'
 	 */
-	public static Time changeTimezone(Connection conn, Time t, TimeZone fromTz,
-			TimeZone toTz, boolean rollForward) {
-		if ((conn != null) && conn.getUseTimezone()) {
-			// Convert the timestamp from GMT to the server's timezone
-			Calendar fromCal = Calendar.getInstance(fromTz);
-			fromCal.setTime(t);
+	public static Time changeTimezone(Connection conn,
+			Calendar sessionCalendar, 
+			Calendar targetCalendar, 
+			Time t, 
+			TimeZone fromTz,
+			TimeZone toTz, 
+			boolean rollForward) {
+		if ((conn != null)) {
+			if (conn.getUseTimezone() &&
+				!conn.getNoTimezoneConversionForTimeType()) {
+				// Convert the timestamp from GMT to the server's timezone
+				Calendar fromCal = Calendar.getInstance(fromTz);
+				fromCal.setTime(t);
+	
+				int fromOffset = fromCal.get(Calendar.ZONE_OFFSET)
+						+ fromCal.get(Calendar.DST_OFFSET);
+				Calendar toCal = Calendar.getInstance(toTz);
+				toCal.setTime(t);
+	
+				int toOffset = toCal.get(Calendar.ZONE_OFFSET)
+						+ toCal.get(Calendar.DST_OFFSET);
+				int offsetDiff = fromOffset - toOffset;
+				long toTime = toCal.getTime().getTime();
+	
+				if (rollForward || (conn.isServerTzUTC() && !conn.isClientTzUTC())) {
+					toTime += offsetDiff;
+				} else {
+					toTime -= offsetDiff;
+				}
+	
+				Time changedTime = new Time(toTime);
+	
+				return changedTime;
+			}  else if (conn.getUseJDBCCompliantTimezoneShift()) {
+				if (targetCalendar != null) {
 
-			int fromOffset = fromCal.get(Calendar.ZONE_OFFSET)
-					+ fromCal.get(Calendar.DST_OFFSET);
-			Calendar toCal = Calendar.getInstance(toTz);
-			toCal.setTime(t);
-
-			int toOffset = toCal.get(Calendar.ZONE_OFFSET)
-					+ toCal.get(Calendar.DST_OFFSET);
-			int offsetDiff = fromOffset - toOffset;
-			long toTime = toCal.getTime().getTime();
-
-			if (rollForward || (conn.isServerTzUTC() && !conn.isClientTzUTC())) {
-				toTime += offsetDiff;
-			} else {
-				toTime -= offsetDiff;
+					Time adjustedTime = new Time( 
+							jdbcCompliantZoneShift(sessionCalendar, 
+									targetCalendar, t));
+					
+					return adjustedTime;
+				}
 			}
-
-			Time changedTime = new Time(toTime);
-
-			return changedTime;
 		}
-
+		
 		return t;
 	}
 
@@ -853,37 +871,89 @@ public class TimeUtil {
 	 * 
 	 * @return the timestamp changed to the timezone 'toTz'
 	 */
-	public static Timestamp changeTimezone(Connection conn, Timestamp tstamp,
-			TimeZone fromTz, TimeZone toTz, boolean rollForward) {
-		if ((conn != null) && conn.getUseTimezone()) {
-			// Convert the timestamp from GMT to the server's timezone
-			Calendar fromCal = Calendar.getInstance(fromTz);
-			fromCal.setTime(tstamp);
+	public static Timestamp changeTimezone(Connection conn, 
+			Calendar sessionCalendar, 
+			Calendar targetCalendar, 
+			Timestamp tstamp,
+			TimeZone fromTz, 
+			TimeZone toTz, 
+			boolean rollForward) {
+		if ((conn != null)) {
+			if (conn.getUseTimezone()) {
+				// Convert the timestamp from GMT to the server's timezone
+				Calendar fromCal = Calendar.getInstance(fromTz);
+				fromCal.setTime(tstamp);
+	
+				int fromOffset = fromCal.get(Calendar.ZONE_OFFSET)
+						+ fromCal.get(Calendar.DST_OFFSET);
+				Calendar toCal = Calendar.getInstance(toTz);
+				toCal.setTime(tstamp);
+	
+				int toOffset = toCal.get(Calendar.ZONE_OFFSET)
+						+ toCal.get(Calendar.DST_OFFSET);
+				int offsetDiff = fromOffset - toOffset;
+				long toTime = toCal.getTime().getTime();
+	
+				if (rollForward || (conn.isServerTzUTC() && !conn.isClientTzUTC())) {
+					toTime += offsetDiff;
+				} else {
+					toTime -= offsetDiff;
+				}
+	
+				Timestamp changedTimestamp = new Timestamp(toTime);
+	
+				return changedTimestamp;
+			} else if (conn.getUseJDBCCompliantTimezoneShift()) {
+				if (targetCalendar != null) {
 
-			int fromOffset = fromCal.get(Calendar.ZONE_OFFSET)
-					+ fromCal.get(Calendar.DST_OFFSET);
-			Calendar toCal = Calendar.getInstance(toTz);
-			toCal.setTime(tstamp);
-
-			int toOffset = toCal.get(Calendar.ZONE_OFFSET)
-					+ toCal.get(Calendar.DST_OFFSET);
-			int offsetDiff = fromOffset - toOffset;
-			long toTime = toCal.getTime().getTime();
-
-			if (rollForward || (conn.isServerTzUTC() && !conn.isClientTzUTC())) {
-				toTime += offsetDiff;
-			} else {
-				toTime -= offsetDiff;
+					Timestamp adjustedTimestamp = new Timestamp( 
+							jdbcCompliantZoneShift(sessionCalendar, 
+									targetCalendar, tstamp));
+					
+					adjustedTimestamp.setNanos(tstamp.getNanos());
+					
+					return adjustedTimestamp;
+				}
 			}
-
-			Timestamp changedTimestamp = new Timestamp(toTime);
-
-			return changedTimestamp;
 		}
-
+		
 		return tstamp;
 	}
 
+	private static long jdbcCompliantZoneShift(Calendar sessionCalendar, 
+			Calendar targetCalendar, 
+			java.util.Date dt) {
+		if (sessionCalendar == null) {
+			sessionCalendar = new GregorianCalendar();
+		}
+		
+		// JDBC spec is not clear whether or not this 
+		// calendar should be immutable, so let's treat
+		// it like it is, for safety
+		
+		java.util.Date origCalDate = targetCalendar.getTime();
+		java.util.Date origSessionDate = sessionCalendar.getTime();
+		
+		try {
+			sessionCalendar.setTime(dt);
+			
+			targetCalendar.set(Calendar.YEAR, sessionCalendar.get(Calendar.YEAR));
+			targetCalendar.set(Calendar.MONTH, sessionCalendar.get(Calendar.MONTH));
+			targetCalendar.set(Calendar.DAY_OF_MONTH, sessionCalendar.get(Calendar.DAY_OF_MONTH));
+
+			targetCalendar.set(Calendar.HOUR_OF_DAY, sessionCalendar.get(Calendar.HOUR_OF_DAY));                
+			targetCalendar.set(Calendar.MINUTE, sessionCalendar.get(Calendar.MINUTE));
+			targetCalendar.set(Calendar.SECOND, sessionCalendar.get(Calendar.SECOND));
+			targetCalendar.set(Calendar.MILLISECOND, sessionCalendar.get(Calendar.MILLISECOND));                
+			
+			return targetCalendar.getTime().getTime();
+			
+		} finally {
+			sessionCalendar.setTime(origSessionDate);
+			targetCalendar.setTime(origCalDate);
+		}
+	}
+
 	//
 	// WARN! You must externally synchronize these calendar instances
 	// See ResultSet.fastDateCreate() for an example
@@ -926,19 +996,19 @@ public class TimeUtil {
 	final static Time fastTimeCreate(Calendar cal, int hour, int minute,
 			int second) throws SQLException {
 		if (hour < 0 || hour > 23) {
-			throw new SQLException("Illegal hour value '" + hour + "' for java.sql.Time type in value '"
+			throw SQLError.createSQLException("Illegal hour value '" + hour + "' for java.sql.Time type in value '"
 					+ timeFormattedString(hour, minute, second) + ".", 
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 		
 		if (minute < 0 || minute > 59) {
-			throw new SQLException("Illegal minute value '" + minute + "'" + "' for java.sql.Time type in value '"
+			throw SQLError.createSQLException("Illegal minute value '" + minute + "'" + "' for java.sql.Time type in value '"
 					+ timeFormattedString(hour, minute, second) + ".", 
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 		
 		if (second < 0 || second > 59) {
-			throw new SQLException("Illegal minute value '" + second + "'" + "' for java.sql.Time type in value '"
+			throw SQLError.createSQLException("Illegal minute value '" + second + "'" + "' for java.sql.Time type in value '"
 					+ timeFormattedString(hour, minute, second) + ".", 
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
@@ -1083,6 +1153,7 @@ public class TimeUtil {
 	
 	private static String timeFormattedString(int hours, int minutes, int seconds) {
 		StringBuffer buf = new StringBuffer(8);
+		
 		if (hours < 10) {
 			buf.append("0");
 		}
