@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -67,6 +67,9 @@ public class UpdatableResultSet extends ResultSet {
 	/** Is this result set updateable? */
 	private boolean isUpdatable = false;
 
+	/** Reason the result set is not updatable */
+	private String notUpdatableReason = null;
+
 	/** List of primary keys */
 	private List primaryKeyIndicies = null;
 
@@ -80,7 +83,7 @@ public class UpdatableResultSet extends ResultSet {
 	private String refreshSQL = null;
 
 	/** The binary data for the 'current' row */
-	private byte[][] savedCurrentRow;
+	private Object[] savedCurrentRow;
 
 	private String tableOnlyName;
 
@@ -89,6 +92,8 @@ public class UpdatableResultSet extends ResultSet {
 
 	/** SQL for in-place modifcation */
 	private String updateSQL = null;
+	
+	private boolean populateInserterWithDefaultValues = false;
 
 	/**
 	 * Create a result set for an executeUpdate statement.
@@ -132,6 +137,8 @@ public class UpdatableResultSet extends ResultSet {
 			Connection conn, Statement creatorStmt) throws SQLException {
 		super(catalog, fields, tuples, conn, creatorStmt);
 		checkUpdatability();
+		this.populateInserterWithDefaultValues = 
+			this.connection.getPopulateInsertRowWithDefaultValues();
 	}
 
 	/**
@@ -246,12 +253,37 @@ public class UpdatableResultSet extends ResultSet {
 	 * @throws SQLException
 	 *             DOCUMENT ME!
 	 */
-	private void checkUpdatability() throws SQLException {
+	protected void checkUpdatability() throws SQLException {
+		if (this.fields == null) {
+			// we've been created to be populated with cached
+			// metadata, and we don't have the metadata yet,
+			// we'll be called again by 
+			// Connection.initializeResultsMetadataFromCache()
+			// when the metadata has been made available
+			
+			return;
+		}
+		
 		String singleTableName = null;
 		String catalogName = null;
 
 		int primaryKeyCount = 0;
 
+		// We can only do this if we know that there is a currently
+		// selected database, or if we're talking to a > 4.1 version
+		// of MySQL server (as it returns database names in field
+		// info)
+		//
+		if ((this.catalog == null) || (this.catalog.length() == 0)) {
+			this.catalog = this.fields[0].getDatabaseName();
+
+			if ((this.catalog == null) || (this.catalog.length() == 0)) {
+				throw SQLError.createSQLException(Messages
+						.getString("UpdatableResultSet.43") //$NON-NLS-1$
+						, SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$ //$NON-NLS-2$
+			}
+		}
+
 		if (this.fields.length > 0) {
 			singleTableName = this.fields[0].getOriginalTableName();
 			catalogName = this.fields[0].getDatabaseName();
@@ -261,6 +293,13 @@ public class UpdatableResultSet extends ResultSet {
 				catalogName = this.catalog;
 			}
 
+			if (singleTableName != null && singleTableName.length() == 0) {
+				this.isUpdatable = false;
+				this.notUpdatableReason = Messages.getString("NotUpdatableReason.3");
+
+				return;
+			}
+			
 			if (this.fields[0].isPrimaryKey()) {
 				primaryKeyCount++;
 			}
@@ -277,9 +316,17 @@ public class UpdatableResultSet extends ResultSet {
 					otherCatalogName = this.catalog;
 				}
 
+				if (otherTableName != null && otherTableName.length() == 0) {
+					this.isUpdatable = false;
+					this.notUpdatableReason = Messages.getString("NotUpdatableReason.3");
+
+					return;
+				}
+				
 				if ((singleTableName == null)
 						|| !otherTableName.equals(singleTableName)) {
 					this.isUpdatable = false;
+					this.notUpdatableReason = Messages.getString("NotUpdatableReason.0");
 
 					return;
 				}
@@ -288,6 +335,7 @@ public class UpdatableResultSet extends ResultSet {
 				if ((catalogName == null)
 						|| !otherCatalogName.equals(catalogName)) {
 					this.isUpdatable = false;
+					this.notUpdatableReason = Messages.getString("NotUpdatableReason.1");
 
 					return;
 				}
@@ -299,39 +347,17 @@ public class UpdatableResultSet extends ResultSet {
 
 			if ((singleTableName == null) || (singleTableName.length() == 0)) {
 				this.isUpdatable = false;
+				this.notUpdatableReason = Messages.getString("NotUpdatableReason.2");
 
 				return;
 			}
 		} else {
 			this.isUpdatable = false;
+			this.notUpdatableReason = Messages.getString("NotUpdatableReason.3");
 
 			return;
 		}
-
-		// 
-		// Must have at least one primary key
-		//
-		if (primaryKeyCount == 0) {
-			this.isUpdatable = false;
-
-			return;
-		}
-
-		// We can only do this if we know that there is a currently
-		// selected database, or if we're talking to a > 4.1 version
-		// of MySQL server (as it returns database names in field
-		// info)
-		//
-		if ((this.catalog == null) || (this.catalog.length() == 0)) {
-			this.catalog = this.fields[0].getDatabaseName();
-
-			if ((this.catalog == null) || (this.catalog.length() == 0)) {
-				throw new SQLException(Messages
-						.getString("UpdatableResultSet.43") //$NON-NLS-1$
-						, SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$ //$NON-NLS-2$
-			}
-		}
-
+		
 		if (this.connection.getStrictUpdates()) {
 			java.sql.DatabaseMetaData dbmd = this.connection.getMetaData();
 
@@ -358,8 +384,11 @@ public class UpdatableResultSet extends ResultSet {
 				}
 			}
 
-			if (primaryKeyNames.size() == 0) {
+			int existingPrimaryKeysCount = primaryKeyNames.size();
+			
+			if (existingPrimaryKeysCount == 0) {
 				this.isUpdatable = false;
+				this.notUpdatableReason = Messages.getString("NotUpdatableReason.5");
 
 				return; // we can't update tables w/o keys
 			}
@@ -381,6 +410,8 @@ public class UpdatableResultSet extends ResultSet {
 									.toUpperCase()) == null) {
 								// we don't know about this key, so give up :(
 								this.isUpdatable = false;
+								this.notUpdatableReason = Messages.getString("NotUpdatableReason.6", 
+										new Object[] {originalName});
 
 								return;
 							}
@@ -390,11 +421,30 @@ public class UpdatableResultSet extends ResultSet {
 			}
 
 			this.isUpdatable = primaryKeyNames.isEmpty();
+			
+			if (!this.isUpdatable) {
+				if (existingPrimaryKeysCount > 1) {
+					this.notUpdatableReason = Messages.getString("NotUpdatableReason.7");
+				} else {
+					this.notUpdatableReason = Messages.getString("NotUpdatableReason.4");
+				}
+				
+				return;
+			}
+		}
+		
+		// 
+		// Must have at least one primary key
+		//
+		if (primaryKeyCount == 0) {
+			this.isUpdatable = false;
+			this.notUpdatableReason = Messages.getString("NotUpdatableReason.4");
 
 			return;
 		}
 
 		this.isUpdatable = true;
+		this.notUpdatableReason = null;
 
 		return;
 	}
@@ -413,17 +463,17 @@ public class UpdatableResultSet extends ResultSet {
 		checkClosed();
 
 		if (!this.isUpdatable) {
-			throw new NotUpdatable();
+			throw new NotUpdatable(this.notUpdatableReason);
 		}
 
 		if (this.onInsertRow) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.1")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.1")); //$NON-NLS-1$
 		} else if (this.rowData.size() == 0) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.2")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.2")); //$NON-NLS-1$
 		} else if (isBeforeFirst()) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.3")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.3")); //$NON-NLS-1$
 		} else if (isAfterLast()) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.4")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.4")); //$NON-NLS-1$
 		}
 
 		if (this.deleter == null) {
@@ -471,7 +521,7 @@ public class UpdatableResultSet extends ResultSet {
 			this.deleter.executeUpdate();
 			this.rowData.removeRow(this.rowData.getCurrentRowNumber());
 		} catch (java.io.UnsupportedEncodingException encodingEx) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.39", //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.39", //$NON-NLS-1$
 					new Object[] { this.charEncoding }) //$NON-NLS-1$
 					, SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$ //$NON-NLS-2$
 		}
@@ -555,7 +605,7 @@ public class UpdatableResultSet extends ResultSet {
 			this.doingUpdates = false;
 			this.onInsertRow = false;
 
-			throw new NotUpdatable();
+			throw new NotUpdatable(this.notUpdatableReason);
 		}
 
 		String quotedId = getQuotedIdChar();
@@ -723,17 +773,11 @@ public class UpdatableResultSet extends ResultSet {
 		checkClosed();
 
 		if (!this.onInsertRow) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.7")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.7")); //$NON-NLS-1$
 		}
 
 		this.inserter.executeUpdate();
 
-		int numPrimaryKeys = 0;
-
-		if (this.primaryKeyIndicies != null) {
-			numPrimaryKeys = this.primaryKeyIndicies.size();
-		}
-
 		long autoIncrementId = this.inserter.getLastInsertID();
 		int numFields = this.fields.length;
 		byte[][] newRow = new byte[numFields][];
@@ -745,16 +789,23 @@ public class UpdatableResultSet extends ResultSet {
 				newRow[i] = this.inserter.getBytesRepresentation(i);
 			}
 
-			if ((numPrimaryKeys == 1) && this.fields[i].isPrimaryKey()
-					&& (autoIncrementId > 0)) {
+			//
+			// WARN: This non-variant only holds if MySQL never allows more
+			// than one auto-increment key (which is the way it is _today_)
+			//
+			if (this.fields[i].isAutoIncrement() && autoIncrementId > 0) {
 				newRow[i] = String.valueOf(autoIncrementId).getBytes();
+				this.inserter.setBytesNoEscapeNoQuotes(i + 1, newRow[i]);
 			}
 		}
-
+		
+		refreshRow(this.inserter, newRow);
+		
 		this.rowData.addRow(newRow);
 		resetInserter();
 	}
 
+
 	/**
 	 * JDBC 2.0
 	 * 
@@ -859,7 +910,7 @@ public class UpdatableResultSet extends ResultSet {
 		checkClosed();
 
 		if (!this.isUpdatable) {
-			throw new NotUpdatable();
+			throw new NotUpdatable(this.notUpdatableReason);
 		}
 
 		if (this.onInsertRow) {
@@ -889,7 +940,7 @@ public class UpdatableResultSet extends ResultSet {
 		checkClosed();
 
 		if (!this.isUpdatable) {
-			throw new NotUpdatable();
+			throw new NotUpdatable(this.notUpdatableReason);
 		}
 
 		if (this.inserter == null) {
@@ -899,7 +950,10 @@ public class UpdatableResultSet extends ResultSet {
 
 			this.inserter = this.connection
 					.clientPrepareStatement(this.insertSQL);
-			extractDefaultValues();
+			if (this.populateInserterWithDefaultValues) {
+				extractDefaultValues();
+			}
+			
 			resetInserter();
 		} else {
 			resetInserter();
@@ -909,56 +963,58 @@ public class UpdatableResultSet extends ResultSet {
 
 		this.onInsertRow = true;
 		this.doingUpdates = false;
-		this.savedCurrentRow = (byte[][]) this.thisRow;
+		this.savedCurrentRow = this.thisRow;
 		this.thisRow = new byte[numFields][];
 
 		for (int i = 0; i < numFields; i++) {
-			if (this.defaultColumnValue[i] != null) {
-				Field f = this.fields[i];
-
-				switch (f.getMysqlType()) {
-				case MysqlDefs.FIELD_TYPE_DATE:
-				case MysqlDefs.FIELD_TYPE_DATETIME:
-				case MysqlDefs.FIELD_TYPE_NEWDATE:
-				case MysqlDefs.FIELD_TYPE_TIME:
-				case MysqlDefs.FIELD_TYPE_TIMESTAMP:
-
-					if (this.defaultColumnValue[i].length > 7
-							&& this.defaultColumnValue[i][0] == (byte) 'C'
-							&& this.defaultColumnValue[i][1] == (byte) 'U'
-							&& this.defaultColumnValue[i][2] == (byte) 'R'
-							&& this.defaultColumnValue[i][3] == (byte) 'R'
-							&& this.defaultColumnValue[i][4] == (byte) 'E'
-							&& this.defaultColumnValue[i][5] == (byte) 'N'
-							&& this.defaultColumnValue[i][6] == (byte) 'T'
-							&& this.defaultColumnValue[i][7] == (byte) '_') {
-						this.inserter.setBytesNoEscapeNoQuotes(i + 1,
-								this.defaultColumnValue[i]);
-
-						break;
-					}
-				default:
-					this.inserter.setBytes(i + 1, this.defaultColumnValue[i],
-							false, false);
-				}
-
-				// This value _could_ be changed from a getBytes(), so we
-				// need a copy....
-				byte[] defaultValueCopy = new byte[this.defaultColumnValue[i].length];
-				System.arraycopy(defaultColumnValue[i], 0, defaultValueCopy, 0,
-						defaultValueCopy.length);
-				this.thisRow[i] = defaultValueCopy;
-			} else {
-				this.inserter.setNull(i + 1, java.sql.Types.NULL);
+			if (!this.populateInserterWithDefaultValues) {
+				this.inserter.setBytesNoEscapeNoQuotes(i + 1, 
+						"DEFAULT".getBytes());
 				this.thisRow[i] = null;
+			} else {
+				if (this.defaultColumnValue[i] != null) {
+					Field f = this.fields[i];
+	
+					switch (f.getMysqlType()) {
+					case MysqlDefs.FIELD_TYPE_DATE:
+					case MysqlDefs.FIELD_TYPE_DATETIME:
+					case MysqlDefs.FIELD_TYPE_NEWDATE:
+					case MysqlDefs.FIELD_TYPE_TIME:
+					case MysqlDefs.FIELD_TYPE_TIMESTAMP:
+	
+						if (this.defaultColumnValue[i].length > 7
+								&& this.defaultColumnValue[i][0] == (byte) 'C'
+								&& this.defaultColumnValue[i][1] == (byte) 'U'
+								&& this.defaultColumnValue[i][2] == (byte) 'R'
+								&& this.defaultColumnValue[i][3] == (byte) 'R'
+								&& this.defaultColumnValue[i][4] == (byte) 'E'
+								&& this.defaultColumnValue[i][5] == (byte) 'N'
+								&& this.defaultColumnValue[i][6] == (byte) 'T'
+								&& this.defaultColumnValue[i][7] == (byte) '_') {
+							this.inserter.setBytesNoEscapeNoQuotes(i + 1,
+									this.defaultColumnValue[i]);
+	
+							break;
+						}
+					default:
+						this.inserter.setBytes(i + 1, this.defaultColumnValue[i],
+								false, false);
+					}
+	
+					// This value _could_ be changed from a getBytes(), so we
+					// need a copy....
+					byte[] defaultValueCopy = new byte[this.defaultColumnValue[i].length];
+					System.arraycopy(defaultColumnValue[i], 0, defaultValueCopy, 0,
+							defaultValueCopy.length);
+					this.thisRow[i] = defaultValueCopy;
+				} else {
+					this.inserter.setNull(i + 1, java.sql.Types.NULL);
+					this.thisRow[i] = null;
+				}
 			}
 		}
 	}
-
-	// ---------------------------------------------------------------------
-	// Updates
-	// ---------------------------------------------------------------------
-
+	
 	/**
 	 * A ResultSet is initially positioned before its first row, the first call
 	 * to next makes the first row the current row; the second call makes the
@@ -1029,6 +1085,10 @@ public class UpdatableResultSet extends ResultSet {
 	 *             if an error occurs.
 	 */
 	protected void realClose(boolean calledExplicitly) throws SQLException {
+		if (this.isClosed) {
+			return;
+		}
+		
 		SQLException sqlEx = null;
 
 		if (this.useUsageAdvisor) {
@@ -1039,14 +1099,15 @@ public class UpdatableResultSet extends ResultSet {
 				String message = Messages.getString("UpdatableResultSet.34"); //$NON-NLS-1$
 
 				this.eventSink.consumeEvent(new ProfilerEvent(
-						ProfilerEvent.TYPE_WARN, "", //$NON-NLS-1$
+						ProfilerEvent.TYPE_WARN,
+						"", //$NON-NLS-1$
 						(this.owningStatement == null) ? "N/A" //$NON-NLS-1$
 								: this.owningStatement.currentCatalog, //$NON-NLS-1$
-						this.connection.getId(),
+						this.connectionId,
 						(this.owningStatement == null) ? (-1)
 								: this.owningStatement.getId(), this.resultId,
-						System.currentTimeMillis(), 0, null,
-						this.pointOfOrigin, message));
+						System.currentTimeMillis(), 0, Constants.MILLIS_I18N,
+						null, this.pointOfOrigin, message));
 			}
 		}
 
@@ -1117,15 +1178,20 @@ public class UpdatableResultSet extends ResultSet {
 		}
 
 		if (this.onInsertRow) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.8")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.8")); //$NON-NLS-1$
 		} else if (this.rowData.size() == 0) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.9")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.9")); //$NON-NLS-1$
 		} else if (isBeforeFirst()) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.10")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.10")); //$NON-NLS-1$
 		} else if (isAfterLast()) {
-			throw new SQLException(Messages.getString("UpdatableResultSet.11")); //$NON-NLS-1$
+			throw SQLError.createSQLException(Messages.getString("UpdatableResultSet.11")); //$NON-NLS-1$
 		}
 
+		refreshRow(this.updater, this.thisRow);
+	}
+	
+	private synchronized void refreshRow(PreparedStatement updateInsertStmt, 
+			Object[] rowToRefresh) throws SQLException {
 		if (this.refresher == null) {
 			if (this.refreshSQL == null) {
 				generateStatements();
@@ -1143,14 +1209,14 @@ public class UpdatableResultSet extends ResultSet {
 			byte[] dataFrom = null;
 			int index = ((Integer) this.primaryKeyIndicies.get(0)).intValue();
 
-			if (!this.doingUpdates) {
-				dataFrom = (byte[]) this.thisRow[index];
+			if (!this.doingUpdates && !this.onInsertRow) {
+				dataFrom = (byte[]) rowToRefresh[index];
 			} else {
-				dataFrom = this.updater.getBytesRepresentation(index);
+				dataFrom = updateInsertStmt.getBytesRepresentation(index);
 
 				// Primary keys not set?
-				if (this.updater.isNull(index) || (dataFrom.length == 0)) {
-					dataFrom = (byte[]) this.thisRow[index];
+				if (updateInsertStmt.isNull(index) || (dataFrom.length == 0)) {
+					dataFrom = (byte[]) rowToRefresh[index];
 				} else {
 					dataFrom = stripBinaryPrefix(dataFrom);
 				}
@@ -1163,13 +1229,13 @@ public class UpdatableResultSet extends ResultSet {
 				int index = ((Integer) this.primaryKeyIndicies.get(i))
 						.intValue();
 
-				if (!this.doingUpdates) {
-					dataFrom = (byte[]) this.thisRow[index];
+				if (!this.doingUpdates && !this.onInsertRow) {
+					dataFrom = (byte[]) rowToRefresh[index];
 				} else {
-					dataFrom = this.updater.getBytesRepresentation(index);
+					dataFrom = updateInsertStmt.getBytesRepresentation(index);
 
 					// Primary keys not set?
-					if (this.updater.isNull(index) || (dataFrom.length == 0)) {
+					if (updateInsertStmt.isNull(index) || (dataFrom.length == 0)) {
 						dataFrom = (byte[]) this.thisRow[index];
 					} else {
 						dataFrom = stripBinaryPrefix(dataFrom);
@@ -1192,13 +1258,13 @@ public class UpdatableResultSet extends ResultSet {
 					byte[] val = rs.getBytes(i + 1);
 
 					if ((val == null) || rs.wasNull()) {
-						this.thisRow[i] = null;
+						rowToRefresh[i] = null;
 					} else {
-						this.thisRow[i] = rs.getBytes(i + 1);
+						rowToRefresh[i] = rs.getBytes(i + 1);
 					}
 				}
 			} else {
-				throw new SQLException(Messages
+				throw SQLError.createSQLException(Messages
 						.getString("UpdatableResultSet.12"), //$NON-NLS-1$
 						SQLError.SQL_STATE_GENERAL_ERROR); //$NON-NLS-1$
 			}
@@ -2211,7 +2277,7 @@ public class UpdatableResultSet extends ResultSet {
 	 */
 	public synchronized void updateRow() throws SQLException {
 		if (!this.isUpdatable) {
-			throw new NotUpdatable();
+			throw new NotUpdatable(this.notUpdatableReason);
 		}
 
 		if (this.doingUpdates) {
@@ -2292,6 +2358,8 @@ public class UpdatableResultSet extends ResultSet {
 	 */
 	public synchronized void updateString(int columnIndex, String x)
 			throws SQLException {
+		checkClosed();
+		
 		if (!this.onInsertRow) {
 			if (!this.doingUpdates) {
 				this.doingUpdates = true;
@@ -2435,4 +2503,4 @@ public class UpdatableResultSet extends ResultSet {
 			java.sql.Timestamp x) throws SQLException {
 		updateTimestamp(findColumn(columnName), x);
 	}
-}
+}
\ No newline at end of file
