@@ -75,7 +75,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setArray(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -97,7 +97,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setAsciiStream(
 						parameterIndex, x, length);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -118,7 +118,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setBigDecimal(
 						parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -140,7 +140,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setBinaryStream(
 						parameterIndex, x, length);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -160,7 +160,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setBlob(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -180,7 +180,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setBoolean(
 						parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -200,7 +200,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setByte(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -220,7 +220,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setBytes(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -242,7 +242,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setCharacterStream(
 						parameterIndex, reader, length);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -262,7 +262,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setClob(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -282,7 +282,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setDate(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -304,7 +304,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setDate(parameterIndex,
 						x, cal);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -324,7 +324,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setDouble(
 						parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -344,7 +344,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setFloat(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -364,7 +364,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt)
 						.setInt(parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -384,7 +384,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setLong(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -404,7 +404,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				return ((PreparedStatement) this.wrappedStmt).getMetaData();
 			}
 
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					"No operations allowed after statement closed",
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		} catch (SQLException sqlEx) {
@@ -425,7 +425,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setNull(parameterIndex,
 						sqlType);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -446,7 +446,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setNull(parameterIndex,
 						sqlType, typeName);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -466,7 +466,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setObject(
 						parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -487,7 +487,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setObject(
 						parameterIndex, x, targetSqlType);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -507,9 +507,9 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 		try {
 			if (this.wrappedStmt != null) {
 				((PreparedStatement) this.wrappedStmt).setObject(
-						parameterIndex, x, scale);
+						parameterIndex, x, targetSqlType, scale);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -530,7 +530,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 						.getParameterMetaData();
 			}
 
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					"No operations allowed after statement closed",
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		} catch (SQLException sqlEx) {
@@ -551,7 +551,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt)
 						.setRef(parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -571,7 +571,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setShort(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -591,7 +591,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setString(
 						parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -611,7 +611,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setTime(parameterIndex,
 						x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -633,7 +633,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setTime(parameterIndex,
 						x, cal);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -654,7 +654,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setTimestamp(
 						parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -676,7 +676,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setTimestamp(
 						parameterIndex, x, cal);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -696,7 +696,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt)
 						.setURL(parameterIndex, x);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -729,7 +729,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				((PreparedStatement) this.wrappedStmt).setUnicodeStream(
 						parameterIndex, x, length);
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -748,7 +748,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 			if (this.wrappedStmt != null) {
 				((PreparedStatement) this.wrappedStmt).addBatch();
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -767,7 +767,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 			if (this.wrappedStmt != null) {
 				((PreparedStatement) this.wrappedStmt).clearParameters();
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
@@ -787,7 +787,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				return ((PreparedStatement) this.wrappedStmt).execute();
 			}
 
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					"No operations allowed after statement closed",
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		} catch (SQLException sqlEx) {
@@ -816,7 +816,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				return rs;
 			}
 
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					"No operations allowed after statement closed",
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		} catch (SQLException sqlEx) {
@@ -840,7 +840,7 @@ public class PreparedStatementWrapper extends StatementWrapper implements
 				return ((PreparedStatement) this.wrappedStmt).executeUpdate();
 			}
 
-			throw new SQLException(
+			throw SQLError.createSQLException(
 					"No operations allowed after statement closed",
 					SQLError.SQL_STATE_GENERAL_ERROR);
 		} catch (SQLException sqlEx) {
