@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2005 MySQL AB
+ Copyright (C) 2005-2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -27,7 +27,9 @@ import com.mysql.jdbc.profiler.ProfilerEvent;
 
 public class LogUtils {
 
-	private static final String LINE_SEPARATOR = System
+    public static final String CALLER_INFORMATION_NOT_AVAILABLE = "Caller information not available";
+
+    private static final String LINE_SEPARATOR = System
 			.getProperty("line.separator");
 
 	private static final int LINE_SEPARATOR_LENGTH = LINE_SEPARATOR.length();
@@ -81,7 +83,13 @@ public class LogUtils {
 				appendLocationInfo = true;
 				
 				break;
-
+				
+			case ProfilerEvent.TYPE_SLOW_QUERY:
+				msgBuf.append("SLOW QUERY");
+				appendLocationInfo = false;
+				
+				break;
+				
 			default:
 				msgBuf.append("UNKNOWN");
 			}
@@ -89,8 +97,10 @@ public class LogUtils {
 			msgBuf.append("] ");
 			msgBuf.append(findCallingClassAndMethod(locationException));
 			msgBuf.append(" duration: ");
-			msgBuf.append(evt.getEventDurationMillis());
-			msgBuf.append(" ms, connection-id: ");
+			msgBuf.append(evt.getEventDuration());
+			msgBuf.append(" ");
+			msgBuf.append(evt.getDurationUnits());
+			msgBuf.append(", connection-id: ");
 			msgBuf.append(evt.getConnectionId());
 			msgBuf.append(", statement-id: ");
 			msgBuf.append(evt.getStatementId());
@@ -115,13 +125,12 @@ public class LogUtils {
 		}
 		
 		return possibleProfilerEvent;
-
 	}
 
 	public static String findCallingClassAndMethod(Throwable t) {
 		String stackTraceAsString = Util.stackTraceToString(t);
 
-		String callingClassAndMethod = "Caller information not available";
+		String callingClassAndMethod = CALLER_INFORMATION_NOT_AVAILABLE;
 
 		int endInternalMethods = stackTraceAsString
 				.lastIndexOf("com.mysql.jdbc");
