@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2002-2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -29,6 +29,9 @@ import testsuite.BaseTestCase;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileWriter;
+import java.io.PrintStream;
+import java.net.InetAddress;
+import java.net.NetworkInterface;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
 import java.sql.PreparedStatement;
@@ -38,10 +41,15 @@ import java.sql.SQLException;
 import java.sql.Savepoint;
 import java.sql.Statement;
 
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.Iterator;
+import java.util.List;
 import java.util.Properties;
 import java.util.StringTokenizer;
 
 import com.mysql.jdbc.ConnectionPropertiesTransform;
+import com.mysql.jdbc.Driver;
 import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.SQLError;
 import com.mysql.jdbc.StringUtils;
@@ -187,7 +195,10 @@ public class ConnectionTest extends BaseTestCase {
 			this.conn.createStatement().executeQuery(
 					"SELECT * FROM t1 WHERE id=0 FOR UPDATE");
 
-			Connection deadlockConn = getConnectionWithProps(new Properties());
+			Properties props = new Properties();
+			props.setProperty("includeInnodbStatusInDeadlockExceptions", "true");
+			
+			Connection deadlockConn = getConnectionWithProps(props);
 			deadlockConn.setAutoCommit(false);
 
 			// The following query should hang because con1 is locking the page
@@ -208,6 +219,8 @@ public class ConnectionTest extends BaseTestCase {
 			//
 			assertTrue(SQLError.SQL_STATE_DEADLOCK.equals(sqlEx.getSQLState()));
 			assertTrue(sqlEx.getErrorCode() == 1205);
+			// Make sure INNODB Status is getting dumped into error message
+			assertTrue(sqlEx.getMessage().indexOf("INNODB MONITOR") != -1);
 		} finally {
 			this.conn.setAutoCommit(true);
 			this.stmt.executeUpdate("DROP TABLE IF EXISTS t1");
@@ -653,7 +666,7 @@ public class ConnectionTest extends BaseTestCase {
 	public void testNonStandardConnectionCollation() throws Exception {
 		if (versionMeetsMinimum(4, 1)) {
 			String collationToSet = "utf8_bin";
-			String characterSet = "utf8";
+			String characterSet = "utf-8";
 
 			Properties props = new Properties();
 			props.setProperty("connectionCollation", collationToSet);
@@ -829,6 +842,36 @@ public class ConnectionTest extends BaseTestCase {
 		}
 	}
 
+	public void testLocalInfileDisabled() throws Exception {
+		createTable("testLocalInfileDisabled", "(field1 varchar(255))");
+		
+		File infile = File.createTempFile("foo", "txt");
+		infile.deleteOnExit();
+		String url = infile.toURL().toExternalForm();
+		FileWriter output = new FileWriter(infile);
+		output.write("Test");
+		output.flush();
+		output.close();
+		
+		Connection loadConn = getConnectionWithProps(new Properties());
+		
+		try {
+			// have to do this after connect, otherwise it's the server
+			// that's enforcing it
+			((com.mysql.jdbc.Connection)loadConn).setAllowLoadLocalInfile(false);
+			try {
+				loadConn.createStatement().execute("LOAD DATA LOCAL INFILE '" + infile.getCanonicalPath() + "' INTO TABLE testLocalInfileDisabled");
+				fail("Should've thrown an exception.");
+			} catch (SQLException sqlEx) {
+				assertEquals(SQLError.SQL_STATE_GENERAL_ERROR, sqlEx.getSQLState());
+			}
+			
+			assertFalse(loadConn.createStatement().executeQuery("SELECT * FROM testLocalInfileDisabled").next());
+		} finally {
+			loadConn.close();
+		}
+	}
+	
 	public void testServerConfigurationCache() throws Exception {
 		Properties props = new Properties();
 
@@ -1091,7 +1134,7 @@ public class ConnectionTest extends BaseTestCase {
 	}
 
 	public void testPing() throws SQLException {
-		Connection conn2 = getConnectionWithProps(null);
+		Connection conn2 = getConnectionWithProps((Properties)null);
 
 		((com.mysql.jdbc.Connection) conn2).ping();
 		conn2.close();
@@ -1200,4 +1243,279 @@ public class ConnectionTest extends BaseTestCase {
         stmt1.close();
         conn1.close();
     }
+    
+    /**
+     * Tests feature of "localSocketAddress", by enumerating local IF's and
+     * trying each one in turn. This test might take a long time to run, since
+     * we can't set timeouts if we're using localSocketAddress. We try and keep
+     * the time down on the testcase by spawning the checking of each interface
+     * off into separate threads.
+     * 
+     * @throws Exception if the test can't use at least one of the local machine's
+     *                   interfaces to make an outgoing connection to the server.
+     */
+    public void testLocalSocketAddress() throws Exception {
+    	if (isRunningOnJdk131()) { 
+    		return;
+    	}
+    	
+    	Enumeration allInterfaces = NetworkInterface.getNetworkInterfaces();
+    	
+    	
+    	SpawnedWorkerCounter counter = new SpawnedWorkerCounter();
+    	
+    	List allChecks = new ArrayList();
+    	
+    	while (allInterfaces.hasMoreElements()) {
+    		NetworkInterface intf = (NetworkInterface)allInterfaces.nextElement();
+    		
+    		Enumeration allAddresses = intf.getInetAddresses();
+
+    		allChecks.add(new LocalSocketAddressCheckThread(allAddresses, counter));
+    	}
+    	
+    	counter.setWorkerCount(allChecks.size());
+    	
+    	for (Iterator it = allChecks.iterator(); it.hasNext();) {
+    		LocalSocketAddressCheckThread t = (LocalSocketAddressCheckThread)it.next();
+    		t.start();
+    	}
+    	
+    	// Wait for tests to complete....
+    	synchronized (counter) {
+    	
+    		while (counter.workerCount > 0 /* safety valve */) {
+    		
+    			counter.wait();
+
+    			if (counter.workerCount == 0) {
+    				System.out.println("Done!");
+    				break;
+    			}
+    		}
+    	}
+    	
+    	boolean didOneWork = false;
+    	boolean didOneFail = false;
+    	
+    	for (Iterator it = allChecks.iterator(); it.hasNext();) {
+    		LocalSocketAddressCheckThread t = (LocalSocketAddressCheckThread)it.next();
+
+    		if (t.atLeastOneWorked) {
+    			didOneWork = true;
+    			
+    			break;
+    		} else {
+    			if (!didOneFail) {
+    				didOneFail = true;
+    			}
+    		}
+    	}
+    	
+    	assertTrue("At least one connection was made with the localSocketAddress set", didOneWork);
+    	
+    	NonRegisteringDriver d = new NonRegisteringDriver();
+    	
+    	String hostname = d.host(d.parseURL(dbUrl, null));
+    	
+    	if (!hostname.startsWith(":") && !hostname.startsWith("localhost")) {
+    		
+    		int indexOfColon = hostname.indexOf(":");
+    		
+    		if (indexOfColon != -1) {
+    			hostname = hostname.substring(0, indexOfColon);
+    		}
+    		
+    		boolean isLocalIf = false;
+    		
+    		isLocalIf = (null != NetworkInterface.getByName(hostname));
+    		
+    		if (!isLocalIf) {
+    			try {
+    				isLocalIf = (null != NetworkInterface.getByInetAddress(InetAddress.getByName(hostname)));
+    			} catch (Throwable t) {
+    				isLocalIf = false;
+    			}
+    		}
+    		
+    		if (!isLocalIf) {
+    			assertTrue("At least one connection didn't fail with localSocketAddress set", didOneFail);
+    		}
+    	}
+    }
+    
+    class SpawnedWorkerCounter {
+    	private int workerCount = 0;
+    	
+    	synchronized void setWorkerCount(int i) {
+    		workerCount = i;
+    	}
+    	
+    	synchronized void decrementWorkerCount() {
+    		workerCount--;
+    		notify();
+    	}
+    }
+    
+    class LocalSocketAddressCheckThread extends Thread {
+    	boolean atLeastOneWorked = false;
+    	Enumeration allAddresses = null;
+    	SpawnedWorkerCounter counter = null;
+    	
+    	LocalSocketAddressCheckThread(Enumeration e, SpawnedWorkerCounter c) {
+    		allAddresses = e;
+    		counter = c;
+    	}
+    	
+    	public void run() {
+    		
+    		while (allAddresses.hasMoreElements()) {
+    			InetAddress addr = (InetAddress)allAddresses.nextElement();
+    			
+    			try {
+    				Properties props = new Properties();
+    				props.setProperty("localSocketAddress", addr.getHostAddress());
+    				props.setProperty("connectTimeout", "2000");
+    				getConnectionWithProps(props).close();
+    				
+    				atLeastOneWorked = true;
+    				
+    				break;
+    			} catch (SQLException sqlEx) {
+    				// ignore, we're only seeing if one of these tests succeeds
+    			}
+    		}
+    		
+    		counter.decrementWorkerCount();
+    	}
+    }
+    
+    public void testUsageAdvisorTooLargeResultSet() throws Exception {
+    	Connection uaConn = null;
+    	
+    	PrintStream stderr = System.err;
+    	
+    	StringBuffer logBuf = new StringBuffer();
+    	
+    	StandardLogger.bufferedLog = logBuf;
+    	
+    	try {
+    		Properties props = new Properties();
+    		props.setProperty("useUsageAdvisor", "true");
+    		props.setProperty("resultSetSizeThreshold", "4");
+    		props.setProperty("logger", "StandardLogger");
+    		
+    		uaConn = getConnectionWithProps(props);
+    		
+    		assertTrue("Result set threshold message not present", 
+    				logBuf.toString().indexOf("larger than \"resultSetSizeThreshold\" of 4 rows") != -1);
+    	} finally {
+    		System.setErr(stderr);
+    		
+    		closeMemberJDBCResources();
+    		
+    		if (uaConn != null) {
+    			uaConn.close();
+    		}
+    	}
+    }
+    
+    public void testUseLocalSessionStateRollback() throws Exception {
+    	if (!versionMeetsMinimum(5, 0, 0)) {
+    		return;
+    	}
+    	
+    	Properties props = new Properties();
+    	props.setProperty("useLocalSessionState", "true");
+    	props.setProperty("profileSQL", "true");
+    	
+    	StringBuffer buf = new StringBuffer();
+    	StandardLogger.bufferedLog = buf;
+    	
+    	createTable("testUseLocalSessionState", "(field1 varchar(32)) ENGINE=InnoDB");
+    	
+    	Connection localStateConn = null;
+    	Statement localStateStmt = null;
+    	
+    	try {
+    		localStateConn = getConnectionWithProps(props);
+        	localStateStmt = localStateConn.createStatement();
+        	
+	    	localStateConn.setAutoCommit(false);
+	    	localStateStmt.executeUpdate("INSERT INTO testUseLocalSessionState VALUES ('abc')");
+	    	localStateConn.rollback();
+	    	localStateConn.rollback();
+	    	localStateStmt.executeUpdate("INSERT INTO testUseLocalSessionState VALUES ('abc')");
+	    	localStateConn.commit();
+	    	localStateConn.commit();
+	    	localStateStmt.close();
+    	} finally {
+    		StandardLogger.bufferedLog = null;
+    		 
+    		if (localStateStmt != null) {
+    			localStateStmt.close();
+    		}
+    		
+    		if (localStateConn != null) {
+    			localStateConn.close();
+    		}
+    	}
+    	
+    	int rollbackCount = 0;
+    	int rollbackPos = 0;
+    	
+    	String searchIn = buf.toString();
+    	
+    	while (rollbackPos != -1) {
+    		rollbackPos = searchIn.indexOf("rollback", rollbackPos);
+    		
+    		if (rollbackPos != -1) {
+    			rollbackPos += "rollback".length();
+    			rollbackCount++;
+    		}
+    	}
+    	
+    	assertEquals(1, rollbackCount);
+    	
+    	int commitCount = 0;
+    	int commitPos = 0;
+    	
+    	// space is important here, we don't want to count "autocommit"
+    	while (commitPos != -1) {
+    		commitPos = searchIn.indexOf(" commit", commitPos);
+    		
+    		if (commitPos != -1) {
+    			commitPos += " commit".length();
+    			commitCount++;
+    		}
+    	}
+    	
+    	assertEquals(1, commitCount);
+    }
+    
+    /**
+     * Checks if setting useCursorFetch to "true" automatically
+     * enables server-side prepared statements.
+     */
+     
+    public void testCouplingOfCursorFetch() throws Exception {
+    	if (!versionMeetsMinimum(5, 0)) {
+    		return;
+    	}
+    	
+    	Connection fetchConn = null;
+    	
+    	try {
+    		Properties props = new Properties();
+    		props.setProperty("useServerPrepStmts", "false"); // force the issue
+    		props.setProperty("useCursorFetch", "true");
+    		fetchConn = getConnectionWithProps(props);
+    		assertEquals("com.mysql.jdbc.ServerPreparedStatement",
+    				fetchConn.prepareStatement("SELECT 1").getClass().getName());
+    	} finally {
+    		if (fetchConn != null) {
+    			fetchConn.close();
+    		}
+    	}
+    }
 }
